@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2013-2017 Heiko Hund <heiko.hund@sophos.com>
+ *  Copyright (C) 2013-2026 Heiko Hund <heiko.hund@sophos.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -16,16 +16,18 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program (see the file COPYING included with this
- *  distribution); if not, write to the Free Software Foundation, Inc.,
- *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef OPENVPN_MSG_H_
 #define OPENVPN_MSG_H_
 
-typedef enum {
+#include <windef.h>
+#include <ws2tcpip.h>
+
+typedef enum
+{
     msg_acknowledgement,
     msg_add_address,
     msg_del_address,
@@ -33,31 +35,49 @@ typedef enum {
     msg_del_route,
     msg_add_dns_cfg,
     msg_del_dns_cfg,
+    msg_add_nrpt_cfg,
+    msg_del_nrpt_cfg,
     msg_add_nbt_cfg,
     msg_del_nbt_cfg,
     msg_flush_neighbors,
-    msg_add_block_dns,
-    msg_del_block_dns,
-    msg_register_dns
+    msg_add_wfp_block,
+    msg_del_wfp_block,
+    msg_register_dns,
+    msg_enable_dhcp,
+    deprecated_msg_register_ring_buffers,
+    msg_set_mtu,
+    msg_add_wins_cfg,
+    msg_del_wins_cfg,
+    msg_create_adapter
 } message_type_t;
 
-typedef struct {
+typedef struct
+{
     message_type_t type;
     size_t size;
     int message_id;
 } message_header_t;
 
-typedef union {
+typedef union
+{
     struct in_addr ipv4;
     struct in6_addr ipv6;
 } inet_address_t;
 
-typedef struct {
+typedef struct
+{
     int index;
     char name[256];
 } interface_t;
 
-typedef struct {
+typedef enum
+{
+    wfp_block_local = 1 << 0,
+    wfp_block_dns = 1 << 1
+} wfp_block_flags_t;
+
+typedef struct
+{
     message_header_t header;
     short family;
     inet_address_t address;
@@ -65,7 +85,8 @@ typedef struct {
     interface_t iface;
 } address_message_t;
 
-typedef struct {
+typedef struct
+{
     message_header_t header;
     short family;
     inet_address_t prefix;
@@ -75,7 +96,8 @@ typedef struct {
     int metric;
 } route_message_t;
 
-typedef struct {
+typedef struct
+{
     message_header_t header;
     interface_t iface;
     char domains[512];
@@ -84,7 +106,35 @@ typedef struct {
     inet_address_t addr[4]; /* support up to 4 dns addresses */
 } dns_cfg_message_t;
 
-typedef struct {
+
+typedef enum
+{
+    nrpt_dnssec
+} nrpt_flags_t;
+
+#define NRPT_ADDR_NUM  8  /* Max. number of addresses */
+#define NRPT_ADDR_SIZE 48 /* Max. address strlen + some */
+typedef char nrpt_address_t[NRPT_ADDR_SIZE];
+typedef struct
+{
+    message_header_t header;
+    interface_t iface;
+    nrpt_address_t addresses[NRPT_ADDR_NUM];
+    char resolve_domains[512]; /* double \0 terminated */
+    char search_domains[512];
+    nrpt_flags_t flags;
+} nrpt_dns_cfg_message_t;
+
+typedef struct
+{
+    message_header_t header;
+    interface_t iface;
+    int addr_len;
+    inet_address_t addr[4]; /* support up to 4 dns addresses */
+} wins_cfg_message_t;
+
+typedef struct
+{
     message_header_t header;
     interface_t iface;
     int disable_nbt;
@@ -96,20 +146,50 @@ typedef struct {
 
 /* TODO: NTP */
 
-typedef struct {
+typedef struct
+{
     message_header_t header;
     short family;
     interface_t iface;
 } flush_neighbors_message_t;
 
-typedef struct {
+typedef struct
+{
     message_header_t header;
     int error_number;
 } ack_message_t;
 
-typedef struct {
+typedef struct
+{
+    message_header_t header;
+    wfp_block_flags_t flags;
+    interface_t iface;
+} wfp_block_message_t;
+
+typedef struct
+{
     message_header_t header;
     interface_t iface;
-} block_dns_message_t;
+} enable_dhcp_message_t;
+
+typedef struct
+{
+    message_header_t header;
+    interface_t iface;
+    short family;
+    int mtu;
+} set_mtu_message_t;
+
+typedef enum
+{
+    ADAPTER_TYPE_DCO,
+    ADAPTER_TYPE_TAP,
+} adapter_type_t;
+
+typedef struct
+{
+    message_header_t header;
+    adapter_type_t adapter_type;
+} create_adapter_message_t;
 
 #endif /* ifndef OPENVPN_MSG_H_ */
