@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -32,27 +31,37 @@
  * outside of the USE_COMP define */
 
 /* Compression flags */
-#define COMP_F_ADAPTIVE             (1<<0) /* COMP_ALG_LZO only */
-#define COMP_F_ALLOW_COMPRESS       (1<<1) /* not only downlink is compressed but also uplink */
-#define COMP_F_SWAP                 (1<<2) /* initial command byte is swapped with last byte in buffer to preserve payload alignment */
-#define COMP_F_ADVERTISE_STUBS_ONLY (1<<3) /* tell server that we only support compression stubs */
-#define COMP_F_ALLOW_STUB_ONLY      (1<<4) /* Only accept stub compression, even with COMP_F_ADVERTISE_STUBS_ONLY
-                                            * we still accept other compressions to be pushed */
-#define COMP_F_MIGRATE              (1<<5) /* push stub-v2 or comp-lzo no when we see a client with comp-lzo in occ */
-#define COMP_F_ALLOW_ASYM           (1<<6) /* Compression was explicitly set to allow asymetric compression */
-#define COMP_F_ALLOW_NOCOMP_ONLY    (1<<7) /* Do not allow compression framing (breaks DCO) */
+/* Removed
+ #define COMP_F_ADAPTIVE             (1u<<0) / * COMP_ALG_LZO only * /
+ #define COMP_F_ALLOW_COMPRESS       (1u<<1) / * not only incoming is compressed but also outgoing *
+ /
+ */
+/** initial command byte is swapped with last byte in buffer to preserve payload alignment */
+#define COMP_F_SWAP                 (1u << 2)
+/** tell server that we only support compression stubs */
+#define COMP_F_ADVERTISE_STUBS_ONLY (1u << 3)
+/** Only accept stub compression, even with COMP_F_ADVERTISE_STUBS_ONLY
+ * we still accept other compressions to be pushed */
+#define COMP_F_ALLOW_STUB_ONLY      (1u << 4)
+/** push stub-v2 or comp-lzo no when we see a client with comp-lzo in occ */
+#define COMP_F_MIGRATE              (1u << 5)
+/** Compression was explicitly set to allow asymetric compression */
+#define COMP_F_ALLOW_ASYM           (1u << 6)
+/** Do not allow compression framing (breaks DCO) */
+#define COMP_F_ALLOW_NOCOMP_ONLY    (1u << 7)
 
 /* algorithms */
 #define COMP_ALG_UNDEF  0
-#define COMP_ALG_STUB   1 /* support compression command byte and framing without actual compression */
-#define COMP_ALG_LZO    2 /* LZO algorithm */
-#define COMP_ALG_SNAPPY 3 /* Snappy algorithm (no longer supported) */
-#define COMP_ALG_LZ4    4 /* LZ4 algorithm */
+/** support compression command byte and framing without actual compression */
+#define COMP_ALG_STUB   1
+#define COMP_ALG_LZO    2 /**< LZO algorithm */
+#define COMP_ALG_SNAPPY 3 /**< Snappy algorithm (no longer supported) */
+#define COMP_ALG_LZ4    4 /**< LZ4 algorithm */
 
 
 /* algorithm v2 */
 #define COMP_ALGV2_UNCOMPRESSED 10
-#define COMP_ALGV2_LZ4      11
+#define COMP_ALGV2_LZ4          11
 /*
  #define COMP_ALGV2_LZO     12
  #define COMP_ALGV2_SNAPPY   13
@@ -71,18 +80,18 @@ struct compress_options
 static inline bool
 comp_non_stub_enabled(const struct compress_options *info)
 {
-    return info->alg != COMP_ALGV2_UNCOMPRESSED
-           && info->alg != COMP_ALG_STUB
+    return info->alg != COMP_ALGV2_UNCOMPRESSED && info->alg != COMP_ALG_STUB
            && info->alg != COMP_ALG_UNDEF;
 }
 
+#include "error.h"
+
 /**
  * Checks if the compression settings are valid. Takes into account the
  * flags of allow-compression and also the whether algorithms are compiled
  * in
  */
-bool
-check_compression_settings_valid(struct compress_options *info, int msglevel);
+bool check_compression_settings_valid(struct compress_options *info, msglvl_t msglevel);
 
 #ifdef USE_COMP
 #include "buffer.h"
@@ -101,17 +110,18 @@ check_compression_settings_valid(struct compress_options *info, int msglevel);
 
 /* V1 on wire codes */
 /* Initial command byte to tell our peer if we compressed */
-#define LZO_COMPRESS_BYTE 0x66
-#define LZ4_COMPRESS_BYTE 0x69
+#define LZO_COMPRESS_BYTE     0x66
+#define LZ4_COMPRESS_BYTE     0x69
 #define NO_COMPRESS_BYTE      0xFA
-#define NO_COMPRESS_BYTE_SWAP 0xFB /* to maintain payload alignment, replace this byte with last byte of packet */
+/** to maintain payload alignment, replace this byte with last byte of packet */
+#define NO_COMPRESS_BYTE_SWAP 0xFB
 
 /* V2 on wire code */
-#define COMP_ALGV2_INDICATOR_BYTE       0x50
-#define COMP_ALGV2_UNCOMPRESSED_BYTE    0
-#define COMP_ALGV2_LZ4_BYTE             1
-#define COMP_ALGV2_LZO_BYTE             2
-#define COMP_ALGV2_SNAPPY_BYTE          3
+#define COMP_ALGV2_INDICATOR_BYTE    0x50
+#define COMP_ALGV2_UNCOMPRESSED_BYTE 0
+#define COMP_ALGV2_LZ4_BYTE          1
+#define COMP_ALGV2_LZO_BYTE          2
+#define COMP_ALGV2_SNAPPY_BYTE       3
 
 /*
  * Compress worst case size expansion (for any algorithm)
@@ -120,7 +130,7 @@ check_compression_settings_valid(struct compress_options *info, int msglevel);
  * Snappy: len + len/6 + 32
  * LZ4:    len + len/255 + 16  (LZ4_COMPRESSBOUND(len))
  */
-#define COMP_EXTRA_BUFFER(len) ((len)/6 + 128 + 3 + COMP_PREFIX_LEN)
+#define COMP_EXTRA_BUFFER(len) ((len) / 6 + 128 + 3 + COMP_PREFIX_LEN)
 
 /*
  * Don't try to compress any packet smaller than this.
@@ -138,12 +148,10 @@ struct compress_alg
     const char *name;
     void (*compress_init)(struct compress_context *compctx);
     void (*compress_uninit)(struct compress_context *compctx);
-    void (*compress)(struct buffer *buf, struct buffer work,
-                     struct compress_context *compctx,
+    void (*compress)(struct buffer *buf, struct buffer work, struct compress_context *compctx,
                      const struct frame *frame);
 
-    void (*decompress)(struct buffer *buf, struct buffer work,
-                       struct compress_context *compctx,
+    void (*decompress)(struct buffer *buf, struct buffer work, struct compress_context *compctx,
                        const struct frame *frame);
 };
 
