@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef EVENT_H
@@ -26,18 +25,17 @@
 
 #include "win32.h"
 #include "sig.h"
-#include "perf.h"
 
 /*
  * rwflags passed to event_ctl and returned by
  * struct event_set_return.
  */
-#define READ_SHIFT      0
-#define WRITE_SHIFT     1
+#define READ_SHIFT  0
+#define WRITE_SHIFT 1
 
-#define EVENT_UNDEF     4
-#define EVENT_READ      (1 << READ_SHIFT)
-#define EVENT_WRITE     (1 << WRITE_SHIFT)
+#define EVENT_UNDEF 4
+#define EVENT_READ  (1u << READ_SHIFT)
+#define EVENT_WRITE (1u << WRITE_SHIFT)
 
 /* event flags returned by io_wait.
  *
@@ -58,29 +56,33 @@
  * signals.
  */
 
-#define SOCKET_SHIFT        0
-#define SOCKET_READ         (1 << (SOCKET_SHIFT + READ_SHIFT))
-#define SOCKET_WRITE        (1 << (SOCKET_SHIFT + WRITE_SHIFT))
-#define TUN_SHIFT           2
-#define TUN_READ            (1 << (TUN_SHIFT + READ_SHIFT))
-#define TUN_WRITE           (1 << (TUN_SHIFT + WRITE_SHIFT))
-#define ERR_SHIFT           4
-#define ES_ERROR            (1 << (ERR_SHIFT + READ_SHIFT))
-#define ES_TIMEOUT          (1 << (ERR_SHIFT + WRITE_SHIFT))
-#define MANAGEMENT_SHIFT    6
-#define MANAGEMENT_READ     (1 << (MANAGEMENT_SHIFT + READ_SHIFT))
-#define MANAGEMENT_WRITE    (1 << (MANAGEMENT_SHIFT + WRITE_SHIFT))
-#define FILE_SHIFT          8
-#define FILE_CLOSED         (1 << (FILE_SHIFT + READ_SHIFT))
-#define DCO_SHIFT           10
-#define DCO_READ            (1 << (DCO_SHIFT + READ_SHIFT))
-#define DCO_WRITE           (1 << (DCO_SHIFT + WRITE_SHIFT))
+#define SOCKET_SHIFT     0
+#define SOCKET_READ      (1 << (SOCKET_SHIFT + READ_SHIFT))
+#define SOCKET_WRITE     (1 << (SOCKET_SHIFT + WRITE_SHIFT))
+#define TUN_SHIFT        2
+#define TUN_READ         (1 << (TUN_SHIFT + READ_SHIFT))
+#define TUN_WRITE        (1 << (TUN_SHIFT + WRITE_SHIFT))
+#define ERR_SHIFT        4
+#define ES_ERROR         (1 << (ERR_SHIFT + READ_SHIFT))
+#define ES_TIMEOUT       (1 << (ERR_SHIFT + WRITE_SHIFT))
+#define MANAGEMENT_SHIFT 6
+#define MANAGEMENT_READ  (1 << (MANAGEMENT_SHIFT + READ_SHIFT))
+#define MANAGEMENT_WRITE (1 << (MANAGEMENT_SHIFT + WRITE_SHIFT))
+#define DCO_SHIFT        10
+#define DCO_READ         (1 << (DCO_SHIFT + READ_SHIFT))
+#define DCO_WRITE        (1 << (DCO_SHIFT + WRITE_SHIFT))
 
 /*
  * Initialization flags passed to event_set_init
  */
-#define EVENT_METHOD_US_TIMEOUT   (1<<0)
-#define EVENT_METHOD_FAST         (1<<1)
+#define EVENT_METHOD_US_TIMEOUT (1 << 0)
+#define EVENT_METHOD_FAST       (1 << 1)
+
+/*
+ * The following constant is used as boundary between integer value
+ * and real addresses when passing arguments to event handlers as (void *)
+ */
+#define MULTI_N ((void *)16) /* upper bound on MTCP_x */
 
 #ifdef _WIN32
 
@@ -88,7 +90,7 @@ typedef const struct rw_handle *event_t;
 
 #define UNDEFINED_EVENT (NULL)
 
-#else  /* ifdef _WIN32 */
+#else /* ifdef _WIN32 */
 
 typedef int event_t;
 
@@ -112,7 +114,8 @@ struct event_set_functions
      * 0 on timeout
      * length of event_set_return if at least 1 event is returned
      */
-    int (*wait)(struct event_set *es, const struct timeval *tv, struct event_set_return *out, int outlen);
+    int (*wait)(struct event_set *es, const struct timeval *tv, struct event_set_return *out,
+                int outlen);
 };
 
 struct event_set_return
@@ -126,6 +129,23 @@ struct event_set
     struct event_set_functions func;
 };
 
+typedef enum
+{
+    EVENT_ARG_MULTI_INSTANCE = 0,
+    EVENT_ARG_LINK_SOCKET,
+} event_arg_t;
+
+/* generic event argument object to pass to event_ctl() */
+struct event_arg
+{
+    event_arg_t type;
+    union
+    {
+        struct multi_instance *mi; /* if type = EVENT_ARG_MULTI_INSTANCE */
+        struct link_socket *sock;  /* if type = EVENT_ARG_LINK_SOCKET */
+    } u;
+};
+
 /*
  * maxevents on input:  desired max number of event_t descriptors
  *                      simultaneously set with event_ctl
@@ -166,9 +186,7 @@ static inline int
 event_wait(struct event_set *es, const struct timeval *tv, struct event_set_return *out, int outlen)
 {
     int ret;
-    perf_push(PERF_IO_WAIT);
     ret = (*es->func.wait)(es, tv, out, outlen);
-    perf_pop();
     return ret;
 }
 
@@ -190,7 +208,7 @@ wait_signal(struct event_set *es, void *arg)
     }
 }
 
-#else  /* ifdef _WIN32 */
+#else /* ifdef _WIN32 */
 
 static inline void
 wait_signal(struct event_set *es, void *arg)
