@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -34,7 +33,7 @@
 #include "mtu.h"
 #include "route.h"
 #include "tun.h"
-#include "socket.h"
+#include "socket_util.h"
 #include "plugin.h"
 #include "manage.h"
 #include "proxy.h"
@@ -65,6 +64,9 @@ struct options_pre_connect
     bool tuntap_options_defined;
     struct tuntap_options tuntap_options;
 
+    const char *ifconfig_local;
+    const char *ifconfig_ipv6_local;
+
     bool routes_defined;
     struct route_option_list *routes;
 
@@ -94,14 +96,21 @@ struct options_pre_connect
 #error "At least one of OpenSSL or mbed TLS needs to be defined."
 #endif
 
+struct local_entry
+{
+    const char *local;
+    const char *port;
+    int proto;
+};
+
 struct connection_entry
 {
+    struct local_list *local_list;
     int proto;
     sa_family_t af;
     const char *local_port;
     bool local_port_defined;
     const char *remote_port;
-    const char *local;
     const char *remote;
     bool remote_float;
     bool bind_defined;
@@ -115,39 +124,40 @@ struct connection_entry
     const char *socks_proxy_port;
     const char *socks_proxy_authfile;
 
-    int tun_mtu;         /* MTU of tun device */
-    int occ_mtu;         /* if non-null, this is the MTU we announce to peers in OCC */
-    int tun_mtu_max;     /* maximum MTU that can be pushed */
+    int tun_mtu;          /* MTU of tun device */
+    int occ_mtu;          /* if non-null, this is the MTU we announce to peers in OCC */
+    int tun_mtu_max;      /* maximum MTU that can be pushed */
 
     bool tun_mtu_defined; /* true if user overriding parm with command line option */
     int tun_mtu_extra;
     bool tun_mtu_extra_defined;
-    int link_mtu;        /* MTU of device over which tunnel packets pass via TCP/UDP */
+    int link_mtu;          /* MTU of device over which tunnel packets pass via TCP/UDP */
     bool link_mtu_defined; /* true if user overriding parm with command line option */
-    int tls_mtu;         /* Maximum MTU for the control channel messages */
+    int tls_mtu;           /* Maximum MTU for the control channel messages */
 
     /* Advanced MTU negotiation and datagram fragmentation options */
-    int mtu_discover_type; /* used if OS supports setting Path MTU discovery options on socket */
+    int mtu_discover_type;          /* used if OS supports setting Path MTU discovery options on socket */
 
-    int fragment;        /* internal fragmentation size */
-    bool fragment_encap; /* true if --fragment had the "mtu" parameter to
-                          * include overhead from IP and TCP/UDP encapsulation */
-    int mssfix;          /* Upper bound on TCP MSS */
-    bool mssfix_default; /* true if --mssfix should use the default parameters */
-    bool mssfix_encap;   /* true if --mssfix had the "mtu" parameter to include
-                          * overhead from IP and TCP/UDP encapsulation */
-    bool mssfix_fixed;   /* use the mssfix value without any encapsulation adjustments */
+    int fragment;                   /* internal fragmentation size */
+    bool fragment_encap;            /* true if --fragment had the "mtu" parameter to
+                                     * include overhead from IP and TCP/UDP encapsulation */
+    int mssfix;                     /* Upper bound on TCP MSS */
+    bool mssfix_default;            /* true if --mssfix should use the default parameters */
+    bool mssfix_encap;              /* true if --mssfix had the "mtu" parameter to include
+                                     * overhead from IP and TCP/UDP encapsulation */
+    bool mssfix_fixed;              /* use the mssfix value without any encapsulation adjustments */
 
-    int explicit_exit_notification; /* Explicitly tell peer when we are exiting via OCC_EXIT or [RESTART] message */
+    int explicit_exit_notification; /* Explicitly tell peer when we are exiting via OCC_EXIT or
+                                       [RESTART] message */
 
-#define CE_DISABLED (1<<0)
-#define CE_MAN_QUERY_PROXY (1<<1)
+#define CE_DISABLED                (1u << 0)
+#define CE_MAN_QUERY_PROXY         (1u << 1)
 #define CE_MAN_QUERY_REMOTE_UNDEF  0
 #define CE_MAN_QUERY_REMOTE_QUERY  1
 #define CE_MAN_QUERY_REMOTE_ACCEPT 2
 #define CE_MAN_QUERY_REMOTE_MOD    3
 #define CE_MAN_QUERY_REMOTE_SKIP   4
-#define CE_MAN_QUERY_REMOTE_MASK   (0x07)
+#define CE_MAN_QUERY_REMOTE_MASK   (0x07u)
 #define CE_MAN_QUERY_REMOTE_SHIFT  (2)
     unsigned int flags;
 
@@ -179,6 +189,13 @@ struct remote_entry
 
 #define CONNECTION_LIST_SIZE 64
 
+struct local_list
+{
+    int capacity;
+    int len;
+    struct local_entry **array;
+};
+
 struct connection_list
 {
     int capacity;
@@ -217,7 +234,8 @@ struct remote_host_store
     char port[RH_PORT_LEN];
 };
 
-enum genkey_type {
+enum genkey_type
+{
     GENKEY_SECRET,
     GENKEY_TLS_CRYPTV2_CLIENT,
     GENKEY_TLS_CRYPTV2_SERVER,
@@ -318,43 +336,41 @@ struct options
 
     bool mtu_test;
 
-#ifdef ENABLE_MEMSTATS
-    char *memstats_fn;
-#endif
-
     bool mlock;
 
-    int keepalive_ping;         /* a proxy for ping/ping-restart */
+    int keepalive_ping; /* a proxy for ping/ping-restart */
     int keepalive_timeout;
 
-    int inactivity_timeout;     /* --inactive */
+    int inactivity_timeout; /* --inactive */
     int64_t inactivity_minimum_bytes;
 
-    int session_timeout;        /* Force-kill session after n seconds */
+    int session_timeout;    /* Force-kill session after n seconds */
 
-    int ping_send_timeout;      /* Send a TCP/UDP ping to remote every n seconds */
-    int ping_rec_timeout;       /* Expect a TCP/UDP ping from remote at least once every n seconds */
-    bool ping_timer_remote;     /* Run ping timer only if we have a remote address */
+    int ping_send_timeout;  /* Send a TCP/UDP ping to remote every n seconds */
+    int ping_rec_timeout;   /* Expect a TCP/UDP ping from remote at least once every n seconds */
+    bool ping_timer_remote; /* Run ping timer only if we have a remote address */
 
 #define PING_UNDEF   0
 #define PING_EXIT    1
 #define PING_RESTART 2
     int ping_rec_timeout_action; /* What action to take on ping_rec_timeout (exit or restart)? */
 
-    bool persist_tun;           /* Don't close/reopen TUN/TAP dev on SIGUSR1 or PING_RESTART */
-    bool persist_local_ip;      /* Don't re-resolve local address on SIGUSR1 or PING_RESTART */
-    bool persist_remote_ip;     /* Don't re-resolve remote address on SIGUSR1 or PING_RESTART */
-    bool persist_key;           /* Don't re-read key files on SIGUSR1 or PING_RESTART */
+    bool persist_tun;            /* Don't close/reopen TUN/TAP dev on SIGUSR1 or PING_RESTART */
+    bool persist_local_ip;       /* Don't re-resolve local address on SIGUSR1 or PING_RESTART */
+    bool persist_remote_ip;      /* Don't re-resolve remote address on SIGUSR1 or PING_RESTART */
 
 #if PASSTOS_CAPABILITY
     bool passtos;
 #endif
 
-    int resolve_retry_seconds;  /* If hostname resolve fails, retry for n seconds */
+    int resolve_retry_seconds; /* If hostname resolve fails, retry for n seconds */
     bool resolve_in_advance;
     const char *ip_remote_hint;
 
     struct tuntap_options tuntap_options;
+    /* DCO is disabled and should not be used as backend driver for the
+     * tun/tap device */
+    bool disable_dco;
 
     /* Misc parms */
     const char *username;
@@ -390,9 +406,6 @@ struct options
     int status_file_version;
     int status_file_update_freq;
 
-    /* optimize TUN/TAP/UDP writes */
-    bool fast_io;
-
     struct compress_options comp;
 
     /* buffer sizes */
@@ -411,13 +424,14 @@ struct options
     const char *route_predown_script;
     const char *route_default_gateway;
     const char *route_ipv6_default_gateway;
+    int route_default_table_id;
     int route_default_metric;
     bool route_noexec;
     int route_delay;
     int route_delay_window;
     bool route_delay_defined;
     struct route_option_list *routes;
-    struct route_ipv6_option_list *routes_ipv6;                 /* IPv6 */
+    struct route_ipv6_option_list *routes_ipv6; /* IPv6 */
     bool block_ipv6;
     bool route_nopull;
     bool route_gateway_via_dhcp;
@@ -452,13 +466,13 @@ struct options
     bool server_defined;
     in_addr_t server_network;
     in_addr_t server_netmask;
-    bool server_ipv6_defined;                           /* IPv6 */
-    struct in6_addr server_network_ipv6;                /* IPv6 */
-    unsigned int server_netbits_ipv6;                   /* IPv6 */
+    bool server_ipv6_defined;            /* IPv6 */
+    struct in6_addr server_network_ipv6; /* IPv6 */
+    unsigned int server_netbits_ipv6;    /* IPv6 */
 
-#define SF_NOPOOL (1<<0)
-#define SF_TCP_NODELAY_HELPER (1<<1)
-#define SF_NO_PUSH_ROUTE_GATEWAY (1<<2)
+#define SF_NOPOOL                (1 << 0)
+#define SF_TCP_NODELAY_HELPER    (1 << 1)
+#define SF_NO_PUSH_ROUTE_GATEWAY (1 << 2)
     unsigned int server_flags;
 
     bool server_bridge_proxy_dhcp;
@@ -477,12 +491,12 @@ struct options
     const char *ifconfig_pool_persist_filename;
     int ifconfig_pool_persist_refresh_freq;
 
-    bool ifconfig_ipv6_pool_defined;                    /* IPv6 */
-    struct in6_addr ifconfig_ipv6_pool_base;            /* IPv6 */
-    int ifconfig_ipv6_pool_netbits;                     /* IPv6 */
+    bool ifconfig_ipv6_pool_defined;         /* IPv6 */
+    struct in6_addr ifconfig_ipv6_pool_base; /* IPv6 */
+    int ifconfig_ipv6_pool_netbits;          /* IPv6 */
 
-    int real_hash_size;
-    int virtual_hash_size;
+    uint32_t real_hash_size;
+    uint32_t virtual_hash_size;
     const char *client_connect_script;
     const char *client_disconnect_script;
     const char *learn_address_script;
@@ -490,10 +504,11 @@ struct options
     const char *client_config_dir;
     bool ccd_exclusive;
     bool disable;
+    const char *override_username;
     int n_bcast_buf;
     int tcp_queue_limit;
     struct iroute *iroutes;
-    struct iroute_ipv6 *iroutes_ipv6;                   /* IPv6 */
+    struct iroute_ipv6 *iroutes_ipv6; /* IPv6 */
     bool push_ifconfig_defined;
     in_addr_t push_ifconfig_local;
     in_addr_t push_ifconfig_remote_netmask;
@@ -501,12 +516,12 @@ struct options
     bool push_ifconfig_constraint_defined;
     in_addr_t push_ifconfig_constraint_network;
     in_addr_t push_ifconfig_constraint_netmask;
-    bool push_ifconfig_ipv4_blocked;                    /* IPv4 */
-    bool push_ifconfig_ipv6_defined;                    /* IPv6 */
-    struct in6_addr push_ifconfig_ipv6_local;           /* IPv6 */
-    int push_ifconfig_ipv6_netbits;                     /* IPv6 */
-    struct in6_addr push_ifconfig_ipv6_remote;          /* IPv6 */
-    bool push_ifconfig_ipv6_blocked;                    /* IPv6 */
+    bool push_ifconfig_ipv4_blocked;           /* IPv4 */
+    bool push_ifconfig_ipv6_defined;           /* IPv6 */
+    struct in6_addr push_ifconfig_ipv6_local;  /* IPv6 */
+    int push_ifconfig_ipv6_netbits;            /* IPv6 */
+    struct in6_addr push_ifconfig_ipv6_remote; /* IPv6 */
+    bool push_ifconfig_ipv6_blocked;           /* IPv6 */
     bool enable_c2c;
     bool duplicate_cn;
 
@@ -540,6 +555,7 @@ struct options
     bool pull; /* client pull of config options from server */
     int push_continuation;
     unsigned int push_option_types_found;
+    unsigned int push_update_options_found; /* tracks which option types have been reset in current PUSH_UPDATE sequence */
     const char *auth_user_pass_file;
     bool auth_user_pass_file_inline;
     struct options_pre_connect *pre_connect;
@@ -552,15 +568,17 @@ struct options
     /* Cipher parms */
     const char *shared_secret_file;
     bool shared_secret_file_inline;
+    bool allow_deprecated_insecure_static_crypto;
     int key_direction;
     const char *ciphername;
-    bool enable_ncp_fallback;      /**< If defined fall back to
-                                   * ciphername if NCP fails */
+    bool enable_ncp_fallback; /**< If defined fall back to
+                               * ciphername if NCP fails */
+    /** The original ncp_ciphers specified by the user in the configuration*/
+    const char *ncp_ciphers_conf;
     const char *ncp_ciphers;
     const char *authname;
     const char *engine;
     struct provider_list providers;
-    bool replay;
     bool mute_replay_warnings;
     int replay_window;
     int replay_time;
@@ -624,8 +642,8 @@ struct options
     int tls_timeout;
 
     /* Data channel key renegotiation parameters */
-    int renegotiate_bytes;
-    int renegotiate_packets;
+    int64_t renegotiate_bytes;
+    int64_t renegotiate_packets;
     int renegotiate_seconds;
     int renegotiate_seconds_min;
 
@@ -633,10 +651,8 @@ struct options
      * within n seconds of handshake initiation. */
     int handshake_window;
 
-#ifdef ENABLE_X509ALTUSERNAME
     /* Field list used to be the username in X509 cert. */
     char *x509_username_field[MAX_PARMS];
-#endif
 
     /* Old key allowed to live n seconds after new key goes active */
     int transition_window;
@@ -658,6 +674,8 @@ struct options
 
     const char *tls_crypt_v2_verify_script;
 
+    int tls_crypt_v2_max_age;
+
     /* Allow only one session */
     bool single_session;
 
@@ -677,17 +695,15 @@ struct options
     bool show_net_up;
     int route_method;
     bool block_outside_dns;
-    enum windows_driver_type windows_driver;
+    enum tun_driver_type windows_driver;
 #endif
 
     bool use_peer_id;
     uint32_t peer_id;
 
-#ifdef HAVE_EXPORT_KEYING_MATERIAL
     /* Keying Material Exporters [RFC 5705] */
     const char *keying_material_exporter_label;
     int keying_material_exporter_length;
-#endif
     /* force using TLS key material export for data channel key generation */
     bool force_key_material_export;
 
@@ -710,50 +726,42 @@ struct options
 /*
  * Option classes.
  */
-#define OPT_P_GENERAL         (1<<0)
-#define OPT_P_UP              (1<<1)
-#define OPT_P_ROUTE           (1<<2)
-#define OPT_P_DHCPDNS         (1<<3)    /* includes ip windows options like */
-#define OPT_P_SCRIPT          (1<<4)
-#define OPT_P_SETENV          (1<<5)
-#define OPT_P_SHAPER          (1<<6)
-#define OPT_P_TIMER           (1<<7)
-#define OPT_P_PERSIST         (1<<8)
-#define OPT_P_PERSIST_IP      (1<<9)
-#define OPT_P_COMP            (1<<10) /* TODO */
-#define OPT_P_MESSAGES        (1<<11)
-#define OPT_P_NCP             (1<<12) /**< Negotiable crypto parameters */
-#define OPT_P_TLS_PARMS       (1<<13) /* TODO */
-#define OPT_P_MTU             (1<<14) /* TODO */
-#define OPT_P_NICE            (1<<15)
-#define OPT_P_PUSH            (1<<16)
-#define OPT_P_INSTANCE        (1<<17) /**< allowed in ccd, client-connect etc*/
-#define OPT_P_CONFIG          (1<<18)
-#define OPT_P_EXPLICIT_NOTIFY (1<<19)
-#define OPT_P_ECHO            (1<<20)
-#define OPT_P_INHERIT         (1<<21)
-#define OPT_P_ROUTE_EXTRAS    (1<<22)
-#define OPT_P_PULL_MODE       (1<<23)
-#define OPT_P_PLUGIN          (1<<24)
-#define OPT_P_SOCKBUF         (1<<25)
-#define OPT_P_SOCKFLAGS       (1<<26)
-#define OPT_P_CONNECTION      (1<<27)
-#define OPT_P_PEER_ID         (1<<28)
-#define OPT_P_INLINE          (1<<29)
-#define OPT_P_PUSH_MTU        (1<<30)
+#define OPT_P_GENERAL         (1u << 0)
+#define OPT_P_UP              (1u << 1)
+#define OPT_P_ROUTE           (1u << 2)
+#define OPT_P_DHCPDNS         (1u << 3) /* includes ip windows options like */
+#define OPT_P_SCRIPT          (1u << 4)
+#define OPT_P_SETENV          (1u << 5)
+#define OPT_P_SHAPER          (1u << 6)
+#define OPT_P_TIMER           (1u << 7)
+#define OPT_P_PERSIST         (1u << 8)
+#define OPT_P_PERSIST_IP      (1u << 9)
+#define OPT_P_COMP            (1u << 10) /* TODO */
+#define OPT_P_MESSAGES        (1u << 11)
+#define OPT_P_NCP             (1u << 12) /**< Negotiable crypto parameters */
+#define OPT_P_TLS_PARMS       (1u << 13) /* TODO */
+#define OPT_P_MTU             (1u << 14) /* TODO */
+#define OPT_P_NICE            (1u << 15)
+#define OPT_P_PUSH            (1u << 16)
+#define OPT_P_INSTANCE        (1u << 17) /**< allowed in ccd, client-connect etc*/
+#define OPT_P_CONFIG          (1u << 18)
+#define OPT_P_EXPLICIT_NOTIFY (1u << 19)
+#define OPT_P_ECHO            (1u << 20)
+#define OPT_P_INHERIT         (1u << 21)
+#define OPT_P_ROUTE_EXTRAS    (1u << 22)
+#define OPT_P_PULL_MODE       (1u << 23)
+#define OPT_P_PLUGIN          (1u << 24)
+#define OPT_P_SOCKBUF         (1u << 25)
+#define OPT_P_SOCKFLAGS       (1u << 26)
+#define OPT_P_CONNECTION      (1u << 27)
+#define OPT_P_PEER_ID         (1u << 28)
+#define OPT_P_INLINE          (1u << 29)
+#define OPT_P_PUSH_MTU        (1u << 30)
+#define OPT_P_ROUTE_TABLE     (1u << 31)
 
-#define OPT_P_DEFAULT   (~(OPT_P_INSTANCE|OPT_P_PULL_MODE))
+#define OPT_P_DEFAULT (~(OPT_P_INSTANCE | OPT_P_PULL_MODE))
 
 #define PULL_DEFINED(opt) ((opt)->pull)
-#define PUSH_DEFINED(opt) ((opt)->push_list)
-
-#ifndef PULL_DEFINED
-#define PULL_DEFINED(opt) (false)
-#endif
-
-#ifndef PUSH_DEFINED
-#define PUSH_DEFINED(opt) (false)
-#endif
 
 #ifdef _WIN32
 #define ROUTE_OPTION_FLAGS(o) ((o)->route_method & ROUTE_METHOD_MASK)
@@ -775,18 +783,107 @@ struct options
 #define MAN_CLIENT_AUTH_ENABLED(opt) (false)
 #endif
 
-void parse_argv(struct options *options,
-                const int argc,
-                char *argv[],
-                const int msglevel,
-                const unsigned int permission_mask,
-                unsigned int *option_types_found,
+/*
+ * some PUSH_UPDATE options
+ */
+#define OPT_P_U_ROUTE         (1 << 0)
+#define OPT_P_U_ROUTE6        (1 << 1)
+#define OPT_P_U_DNS           (1 << 2)
+#define OPT_P_U_DHCP          (1 << 3)
+#define OPT_P_U_REDIR_GATEWAY (1 << 4)
+
+struct pull_filter
+{
+#define PUF_TYPE_UNDEF  0 /**< undefined filter type */
+#define PUF_TYPE_ACCEPT 1 /**< filter type to accept a matching option */
+#define PUF_TYPE_IGNORE 2 /**< filter type to ignore a matching option */
+#define PUF_TYPE_REJECT 3 /**< filter type to reject and trigger SIGUSR1 */
+    int type;
+    size_t size;
+    char *pattern;
+    struct pull_filter *next;
+};
+
+struct pull_filter_list
+{
+    struct pull_filter *head;
+    struct pull_filter *tail;
+};
+
+void add_option(struct options *options, char *p[], bool is_inline, const char *file,
+                int line, const int level, const msglvl_t msglevel,
+                const unsigned int permission_mask, unsigned int *option_types_found,
                 struct env_set *es);
 
+/**
+ * @brief Resets options found in the PUSH_UPDATE message that are preceded by the `-` flag.
+ *        This function is used in push-updates to reset specified options.
+ *        The number of parameters `p` must always be 1. If the permission is verified,
+ *        all related options are erased or reset to their default values.
+ *        Upon successful permission verification (by VERIFY_PERMISSION()),
+ *        `option_types_found` is filled with the flag corresponding to the option.
+ *
+ * @param c The context structure.
+ * @param options A pointer to the options structure.
+ * @param p An array of strings containing the options and their parameters.
+ * @param is_inline A boolean indicating if the option is inline.
+ * @param file The file where the function is called.
+ * @param line The line number where the function is called.
+ * @param msglevel The message level.
+ * @param permission_mask The permission mask used by VERIFY_PERMISSION().
+ * @param option_types_found A pointer to the variable where the flags corresponding to the options
+ * found are stored.
+ * @param es The environment set structure.
+ */
+void remove_option(struct context *c, struct options *options, char *p[], bool is_inline,
+                   const char *file, int line, const msglvl_t msglevel,
+                   const unsigned int permission_mask, unsigned int *option_types_found,
+                   struct env_set *es);
+
+/**
+ * @brief Processes an option to update. It first checks whether it has already
+ *        received an option of the same type within the same update message.
+ *        If the option has already been received, it calls add_option().
+ *        Otherwise, it deletes all existing values related to that option before calling
+ * add_option().
+ *
+ * @param c The context structure.
+ * @param options A pointer to the options structure.
+ * @param p An array of strings containing the options and their parameters.
+ * @param is_inline A boolean indicating if the option is inline.
+ * @param file The file where the function is called.
+ * @param line The line number where the function is called.
+ * @param level The level of the option.
+ * @param msglevel The message level for logging.
+ * @param permission_mask The permission mask used by VERIFY_PERMISSION().
+ * @param option_types_found A pointer to the variable where the flags corresponding to the options
+ * found are stored.
+ * @param es The environment set structure.
+ */
+void update_option(struct context *c, struct options *options, char *p[], bool is_inline,
+                   const char *file, int line, const int level, const msglvl_t msglevel,
+                   const unsigned int permission_mask, unsigned int *option_types_found,
+                   struct env_set *es);
+
+void parse_argv(struct options *options, const int argc, char *argv[], const msglvl_t msglevel,
+                const unsigned int permission_mask, unsigned int *option_types_found,
+                struct env_set *es);
+
+void read_config_file(struct options *options, const char *file, int level, const char *top_file,
+                      const int top_line, const msglvl_t msglevel,
+                      const unsigned int permission_mask, unsigned int *option_types_found,
+                      struct env_set *es);
+
+void read_config_string(const char *prefix, struct options *options, const char *config,
+                        const msglvl_t msglevel, const unsigned int permission_mask,
+                        unsigned int *option_types_found, struct env_set *es);
+
 void notnull(const char *arg, const char *description);
 
 void usage_small(void);
 
+void usage(void);
+
 void show_library_versions(const unsigned int flags);
 
 #ifdef _WIN32
@@ -808,12 +905,8 @@ bool string_defined_equal(const char *s1, const char *s2);
 
 const char *options_string_version(const char *s, struct gc_arena *gc);
 
-char *options_string(const struct options *o,
-                     const struct frame *frame,
-                     struct tuntap *tt,
-                     openvpn_net_ctx_t *ctx,
-                     bool remote,
-                     struct gc_arena *gc);
+char *options_string(const struct options *o, const struct frame *frame, struct tuntap *tt,
+                     openvpn_net_ctx_t *ctx, bool remote, struct gc_arena *gc);
 
 bool options_cmp_equal_safe(char *actual, const char *expected, size_t actual_n);
 
@@ -833,8 +926,8 @@ void options_warning(char *actual, const char *expected);
  * @return gc-allocated value of option with name opt_name if option was found,
  *         or NULL otherwise.
  */
-char *options_string_extract_option(const char *options_string,
-                                    const char *opt_name, struct gc_arena *gc);
+char *options_string_extract_option(const char *options_string, const char *opt_name,
+                                    struct gc_arena *gc);
 
 
 void options_postprocess(struct options *options, struct env_set *es);
@@ -843,38 +936,28 @@ bool options_postprocess_pull(struct options *o, struct env_set *es);
 
 void pre_connect_restore(struct options *o, struct gc_arena *gc);
 
-bool apply_push_options(struct options *options,
-                        struct buffer *buf,
-                        unsigned int permission_mask,
-                        unsigned int *option_types_found,
-                        struct env_set *es);
+bool apply_push_options(struct context *c, struct options *options, struct buffer *buf,
+                        unsigned int permission_mask, unsigned int *option_types_found,
+                        struct env_set *es, bool is_update);
 
 void options_detach(struct options *o);
 
-void options_server_import(struct options *o,
-                           const char *filename,
-                           int msglevel,
-                           unsigned int permission_mask,
-                           unsigned int *option_types_found,
+void options_server_import(struct options *o, const char *filename, msglvl_t msglevel,
+                           unsigned int permission_mask, unsigned int *option_types_found,
                            struct env_set *es);
 
 void pre_pull_default(struct options *o);
 
 void rol_check_alloc(struct options *options);
 
-int parse_line(const char *line,
-               char *p[],
-               const int n,
-               const char *file,
-               const int line_num,
-               int msglevel,
-               struct gc_arena *gc);
+int parse_line(const char *line, char *p[], const int n, const char *file, const int line_num,
+               msglvl_t msglevel, struct gc_arena *gc);
 
 /*
  * parse/print topology coding
  */
 
-int parse_topology(const char *str, const int msglevel);
+int parse_topology(const char *str, const msglvl_t msglevel);
 
 const char *print_topology(const int topology);
 
@@ -888,19 +971,18 @@ const char *print_topology(const int topology);
 
 int auth_retry_get(void);
 
-bool auth_retry_set(const int msglevel, const char *option);
+bool auth_retry_set(const msglvl_t msglevel, const char *option);
 
 const char *auth_retry_print(void);
 
-void options_string_import(struct options *options,
-                           const char *config,
-                           const int msglevel,
-                           const unsigned int permission_mask,
-                           unsigned int *option_types_found,
+void options_string_import(struct options *options, const char *config, const msglvl_t msglevel,
+                           const unsigned int permission_mask, unsigned int *option_types_found,
                            struct env_set *es);
 
 bool key_is_external(const struct options *options);
 
+bool has_udp_in_local_list(const struct options *options);
+
 /**
  * Returns whether the current configuration has dco enabled.
  */
@@ -908,7 +990,7 @@ static inline bool
 dco_enabled(const struct options *o)
 {
 #ifdef ENABLE_DCO
-    return !o->tuntap_options.disable_dco;
+    return !o->disable_dco;
 #else
     return false;
 #endif /* ENABLE_DCO */
