@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef PROXY_H
@@ -31,23 +30,25 @@
 #define HTTP_AUTH_NONE   0
 #define HTTP_AUTH_BASIC  1
 #define HTTP_AUTH_DIGEST 2
-#define HTTP_AUTH_NTLM   3
+/* #define HTTP_AUTH_NTLM   3 removed in OpenVPN 2.7 */
 #define HTTP_AUTH_NTLM2  4
 #define HTTP_AUTH_N      5 /* number of HTTP_AUTH methods */
 
-struct http_custom_header {
+struct http_custom_header
+{
     const char *name;
     const char *content;
 };
 
 #define MAX_CUSTOM_HTTP_HEADER 10
-struct http_proxy_options {
+struct http_proxy_options
+{
     const char *server;
     const char *port;
 
-#define PAR_NO  0   /* don't support any auth retries */
-#define PAR_ALL 1   /* allow all proxy auth protocols */
-#define PAR_NCT 2   /* disable cleartext proxy auth protocols */
+#define PAR_NO  0 /* don't support any auth retries */
+#define PAR_ALL 1 /* allow all proxy auth protocols */
+#define PAR_NCT 2 /* disable cleartext proxy auth protocols */
     int auth_retry;
 
     const char *auth_method_string;
@@ -57,17 +58,13 @@ struct http_proxy_options {
     const char *user_agent;
     struct http_custom_header custom_headers[MAX_CUSTOM_HTTP_HEADER];
     bool inline_creds; /* auth_file_up is inline credentials */
-    bool first_time; /* indicates if we need to wipe user creds at the first iteration of the main loop */
+    bool first_time;   /* indicates if we need to wipe user creds at the first iteration of the main
+                          loop */
     bool nocache;
 };
 
-struct http_proxy_options_simple {
-    const char *server;
-    const char *port;
-    int auth_retry;
-};
-
-struct http_proxy_info {
+struct http_proxy_info
+{
     bool defined;
     int auth_method;
     struct http_proxy_options options;
@@ -83,13 +80,17 @@ struct http_proxy_info *http_proxy_new(const struct http_proxy_options *o);
 
 void http_proxy_close(struct http_proxy_info *hp);
 
+bool proxy_recv_char(uint8_t *c, const char *name, socket_descriptor_t sd,
+                     struct timeval *timeout, volatile int *signal_received);
+
+bool proxy_send(socket_descriptor_t sd, const void *buf, size_t buf_len);
+
 bool establish_http_proxy_passthru(struct http_proxy_info *p,
-                                   socket_descriptor_t sd,  /* already open to proxy */
-                                   const char *host,        /* openvpn server remote */
-                                   const char *port,          /* openvpn server port */
+                                   socket_descriptor_t sd, /* already open to proxy */
+                                   const char *host,       /* openvpn server remote */
+                                   const char *port,       /* openvpn server port */
                                    struct event_timeout *server_poll_timeout,
-                                   struct buffer *lookahead,
-                                   struct signal_info *sig_info);
+                                   struct buffer *lookahead, struct signal_info *sig_info);
 
 uint8_t *make_base64_string2(const uint8_t *str, int str_len, struct gc_arena *gc);
 
