@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Technologies, Inc. <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,7 +67,7 @@ system_error_message(int stat, struct gc_arena *gc)
 #ifdef _WIN32
         case OPENVPN_EXECVE_ERROR:
             buf_printf(&out, "external program did not execute -- ");
-        /* fall through */
+            /* fall through */
 
         default:
             buf_printf(&out, "returned error code %d", stat);
@@ -106,6 +105,43 @@ system_error_message(int stat, struct gc_arena *gc)
     return (const char *)out.data;
 }
 
+#ifndef WIN32
+bool
+openvpn_waitpid_check(pid_t pid, const char *msg_prefix, msglvl_t msglevel)
+{
+    if (pid == 0)
+    {
+        return false;
+    }
+    int status;
+    pid_t pidret = waitpid(pid, &status, WNOHANG);
+    if (pidret != pid)
+    {
+        return true;
+    }
+
+    if (WIFEXITED(status))
+    {
+        int exitcode = WEXITSTATUS(status);
+
+        if (exitcode == OPENVPN_EXECVE_FAILURE)
+        {
+            msg(msglevel, "%scould not execute external program (exit code 127)", msg_prefix);
+        }
+        else
+        {
+            msg(msglevel, "%sexternal program exited with error status: %d", msg_prefix, exitcode);
+        }
+    }
+    else if (WIFSIGNALED(status))
+    {
+        msg(msglevel, "%sexternal program received signal %d", msg_prefix, WTERMSIG(status));
+    }
+
+    return false;
+}
+#endif /* ifndef WIN32 */
+
 bool
 openvpn_execve_allowed(const unsigned int flags)
 {
@@ -155,6 +191,10 @@ openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned in
             {
                 msg(M_ERR, "openvpn_execve: unable to fork");
             }
+            else if (flags & S_NOWAITPID)
+            {
+                ret = pid;
+            }
             else /* parent side */
             {
                 if (waitpid(pid, &ret, 0) != pid)
@@ -190,7 +230,8 @@ openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned in
  * Wrapper around openvpn_execve
  */
 int
-openvpn_execve_check(const struct argv *a, const struct env_set *es, const unsigned int flags, const char *error_message)
+openvpn_execve_check(const struct argv *a, const struct env_set *es, const unsigned int flags,
+                     const char *error_message)
 {
     struct gc_arena gc = gc_new();
     const int stat = openvpn_execve(a, es, flags);
@@ -204,6 +245,11 @@ openvpn_execve_check(const struct argv *a, const struct env_set *es, const unsig
             goto done;
         }
     }
+    else if (flags & S_NOWAITPID && (stat > 0))
+    {
+        ret = stat;
+        goto done;
+    }
     else if (platform_system_ok(stat))
     {
         ret = true;
@@ -211,8 +257,7 @@ openvpn_execve_check(const struct argv *a, const struct env_set *es, const unsig
     }
     if (error_message)
     {
-        msg(((flags & S_FATAL) ? M_FATAL : M_WARN), "%s: %s",
-            error_message,
+        msg(((flags & S_FATAL) ? M_FATAL : M_WARN), "%s: %s", error_message,
             system_error_message(stat, &gc));
     }
 done:
@@ -227,7 +272,7 @@ done:
  * associated with formatting and parsing a command line.
  */
 int
-openvpn_popen(const struct argv *a,  const struct env_set *es)
+openvpn_popen(const struct argv *a, const struct env_set *es)
 {
     struct gc_arena gc = gc_new();
     int ret = -1;
@@ -249,20 +294,20 @@ openvpn_popen(const struct argv *a,  const struct env_set *es)
                 pid = fork();
                 if (pid == (pid_t)0)       /* child side */
                 {
-                    close(pipe_stdout[0]);         /* Close read end */
+                    close(pipe_stdout[0]); /* Close read end */
                     dup2(pipe_stdout[1], 1);
                     execve(cmd, argv, envp);
                     exit(OPENVPN_EXECVE_FAILURE);
                 }
-                else if (pid > (pid_t)0)       /* parent side */
+                else if (pid > (pid_t)0) /* parent side */
                 {
                     int status = 0;
 
-                    close(pipe_stdout[1]);        /* Close write end */
+                    close(pipe_stdout[1]); /* Close write end */
                     waitpid(pid, &status, 0);
                     ret = pipe_stdout[0];
                 }
-                else       /* fork failed */
+                else /* fork failed */
                 {
                     close(pipe_stdout[0]);
                     close(pipe_stdout[1]);
