@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,8 +17,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -34,6 +33,9 @@
 
 #ifdef _WIN32
 
+#include <minwindef.h>
+#include <winsock2.h>
+
 #include "buffer.h"
 #include "error.h"
 #include "mtu.h"
@@ -47,7 +49,7 @@
 
 #include <versionhelpers.h>
 
-#include "block_dns.h"
+#include "wfp_block.h"
 
 /*
  * WFP handle
@@ -99,8 +101,7 @@ static char *win_sys_path = NULL; /* GLOBAL */
 /**
  * Set OpenSSL environment variables to a safe directory
  */
-static void
-set_openssl_env_vars();
+static void set_openssl_env_vars(void);
 
 void
 init_win32(void)
@@ -165,9 +166,7 @@ init_security_attributes_allow_all(struct security_attributes *obj)
 }
 
 void
-overlapped_io_init(struct overlapped_io *o,
-                   const struct frame *frame,
-                   BOOL event_state)
+overlapped_io_init(struct overlapped_io *o, const struct frame *frame, BOOL event_state)
 {
     CLEAR(*o);
 
@@ -217,7 +216,8 @@ overlapped_io_state_ascii(const struct overlapped_io *o)
  */
 
 void
-init_net_event_win32(struct rw_handle *event, long network_events, socket_descriptor_t sd, unsigned int flags)
+init_net_event_win32(struct rw_handle *event, long network_events, socket_descriptor_t sd,
+                     unsigned int flags)
 {
     /* manual reset events, initially set to unsignaled */
 
@@ -290,7 +290,8 @@ close_net_event_win32(struct rw_handle *event, socket_descriptor_t sd, unsigned
         {
             if (!CloseHandle(event->read))
             {
-                msg(M_WARN | M_ERRNO, "Warning: CloseHandle (read) failed in close_net_event_win32");
+                msg(M_WARN | M_ERRNO,
+                    "Warning: CloseHandle (read) failed in close_net_event_win32");
             }
             event->read = NULL;
         }
@@ -306,7 +307,8 @@ close_net_event_win32(struct rw_handle *event, socket_descriptor_t sd, unsigned
         {
             if (!CloseHandle(event->write))
             {
-                msg(M_WARN | M_ERRNO, "Warning: CloseHandle (write) failed in close_net_event_win32");
+                msg(M_WARN | M_ERRNO,
+                    "Warning: CloseHandle (write) failed in close_net_event_win32");
             }
             event->write = NULL;
         }
@@ -330,7 +332,7 @@ net_event_win32_start(struct net_event_win32 *ne, long network_events, socket_de
     ASSERT(!socket_defined(ne->sd));
     ne->sd = sd;
     ne->event_mask = 0;
-    init_net_event_win32(&ne->handle, network_events, sd, NE32_PERSIST_EVENT|NE32_WRITE_EVENT);
+    init_net_event_win32(&ne->handle, network_events, sd, NE32_PERSIST_EVENT | NE32_WRITE_EVENT);
 }
 
 void
@@ -404,7 +406,7 @@ win_trigger_event(struct win32_signal *ws)
         ir.Event.KeyEvent.bKeyDown = true;
         if (!stdin_handle || !WriteConsoleInput(stdin_handle, &ir, 1, &tmp))
         {
-            msg(M_WARN|M_ERRNO, "WARN: win_trigger_event: WriteConsoleInput");
+            msg(M_WARN | M_ERRNO, "WARN: win_trigger_event: WriteConsoleInput");
         }
     }
 }
@@ -415,7 +417,7 @@ win_trigger_event(struct win32_signal *ws)
 static bool WINAPI
 win_ctrl_handler(DWORD signum)
 {
-    msg(D_LOW, "win_ctrl_handler: signal received (code=%lu)", (unsigned long) signum);
+    msg(D_LOW, "win_ctrl_handler: signal received (code=%lu)", (unsigned long)signum);
 
     if (siginfo_static.signal_received == SIGTERM)
     {
@@ -433,7 +435,7 @@ win_ctrl_handler(DWORD signum)
             break;
 
         default:
-            msg(D_LOW, "win_ctrl_handler: signal (code=%lu) not handled", (unsigned long) signum);
+            msg(D_LOW, "win_ctrl_handler: signal (code=%lu) not handled", (unsigned long)signum);
             break;
     }
     /* pass all other signals to the next handler */
@@ -447,9 +449,7 @@ win32_signal_clear(struct win32_signal *ws)
 }
 
 void
-win32_signal_open(struct win32_signal *ws,
-                  int force,
-                  const char *exit_event_name,
+win32_signal_open(struct win32_signal *ws, int force, const char *exit_event_name,
                   bool exit_event_initial_state)
 {
     CLEAR(*ws);
@@ -471,12 +471,10 @@ win32_signal_open(struct win32_signal *ws,
             if (GetConsoleMode(ws->in.read, &ws->console_mode_save))
             {
                 /* running on a console */
-                const DWORD new_console_mode = ws->console_mode_save
-                                               & ~(ENABLE_WINDOW_INPUT
-                                                   | ENABLE_PROCESSED_INPUT
-                                                   | ENABLE_LINE_INPUT
-                                                   | ENABLE_ECHO_INPUT
-                                                   | ENABLE_MOUSE_INPUT);
+                const DWORD new_console_mode =
+                    ws->console_mode_save
+                    & ~(ENABLE_WINDOW_INPUT | ENABLE_PROCESSED_INPUT | ENABLE_LINE_INPUT
+                        | ENABLE_ECHO_INPUT | ENABLE_MOUSE_INPUT);
 
                 if (new_console_mode != ws->console_mode_save)
                 {
@@ -499,8 +497,8 @@ win32_signal_open(struct win32_signal *ws,
      * If console open failed, assume we are running
      * as a service.
      */
-    if ((force == WSO_NOFORCE || force == WSO_FORCE_SERVICE)
-        && !HANDLE_DEFINED(ws->in.read) && exit_event_name)
+    if ((force == WSO_NOFORCE || force == WSO_FORCE_SERVICE) && !HANDLE_DEFINED(ws->in.read)
+        && exit_event_name)
     {
         struct security_attributes sa;
         struct gc_arena gc = gc_new();
@@ -511,11 +509,11 @@ win32_signal_open(struct win32_signal *ws,
             msg(M_ERR, "Error: win32_signal_open: init SA failed");
         }
 
-        ws->in.read = CreateEventW(&sa.sa, TRUE, exit_event_initial_state ? TRUE : FALSE,
-                                   exit_event_nameW);
+        ws->in.read =
+            CreateEventW(&sa.sa, TRUE, exit_event_initial_state ? TRUE : FALSE, exit_event_nameW);
         if (ws->in.read == NULL)
         {
-            msg(M_WARN|M_ERRNO, "NOTE: CreateEventW '%s' failed", exit_event_name);
+            msg(M_WARN | M_ERRNO, "NOTE: CreateEventW '%s' failed", exit_event_name);
         }
         else
         {
@@ -531,9 +529,9 @@ win32_signal_open(struct win32_signal *ws,
         gc_free(&gc);
     }
     /* set the ctrl handler in both console and service modes */
-    if (!SetConsoleCtrlHandler((PHANDLER_ROUTINE) win_ctrl_handler, true))
+    if (!SetConsoleCtrlHandler((PHANDLER_ROUTINE)win_ctrl_handler, true))
     {
-        msg(M_WARN|M_ERRNO, "WARN: SetConsoleCtrlHandler failed");
+        msg(M_WARN | M_ERRNO, "WARN: SetConsoleCtrlHandler failed");
     }
 }
 
@@ -560,8 +558,7 @@ keyboard_ir_to_key(INPUT_RECORD *ir)
         return ir->Event.KeyEvent.wVirtualScanCode;
     }
 
-    if ((ir->Event.KeyEvent.dwControlKeyState
-         & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED))
+    if ((ir->Event.KeyEvent.dwControlKeyState & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED))
         && (ir->Event.KeyEvent.wVirtualKeyCode != 18))
     {
         return ir->Event.KeyEvent.wVirtualScanCode * 256;
@@ -623,8 +620,7 @@ win32_service_interrupt(struct win32_signal *ws)
 {
     if (ws->mode == WSO_MODE_SERVICE)
     {
-        if (HANDLE_DEFINED(ws->in.read)
-            && WaitForSingleObject(ws->in.read, 0) == WAIT_OBJECT_0)
+        if (HANDLE_DEFINED(ws->in.read) && WaitForSingleObject(ws->in.read, 0) == WAIT_OBJECT_0)
         {
             return true;
         }
@@ -671,7 +667,7 @@ win32_signal_get(struct win32_signal *ws)
     }
     if (ret)
     {
-        throw_signal(ret); /* this will update signinfo_static.signal received */
+        throw_signal(ret); /* this will update siginfo_static.signal received */
     }
     return (siginfo_static.signal_received);
 }
@@ -681,7 +677,7 @@ win32_pause(struct win32_signal *ws)
 {
     if (ws->mode == WSO_MODE_CONSOLE && HANDLE_DEFINED(ws->in.read))
     {
-        msg(M_INFO|M_NOPREFIX, "Press any key to continue...");
+        msg(M_INFO | M_NOPREFIX, "Press any key to continue...");
         do
         {
             WaitForSingleObject(ws->in.read, INFINITE);
@@ -732,7 +728,8 @@ window_title_generate(const char *title)
     {
         title = "";
     }
-    buf_printf(&out, "[%s] " PACKAGE_NAME " " PACKAGE_VERSION " F4:EXIT F1:USR1 F2:USR2 F3:HUP", title);
+    buf_printf(&out, "[%s] " PACKAGE_NAME " " PACKAGE_VERSION " F4:EXIT F1:USR1 F2:USR2 F3:HUP",
+               title);
     SetConsoleTitle(BSTR(&out));
     gc_free(&gc);
 }
@@ -761,7 +758,7 @@ semaphore_open(struct semaphore *s, const char *name)
 
     if (s->hand == NULL)
     {
-        msg(M_WARN|M_ERRNO, "WARNING: Cannot create Win32 semaphore '%s'", name);
+        msg(M_WARN | M_ERRNO, "WARNING: Cannot create Win32 semaphore '%s'", name);
     }
     else
     {
@@ -779,9 +776,10 @@ semaphore_lock(struct semaphore *s, int timeout_milliseconds)
         DWORD status;
         ASSERT(!s->locked);
 
-        dmsg(D_SEMAPHORE_LOW, "Attempting to lock Win32 semaphore '%s' prior to net shell command (timeout = %d sec)",
-             s->name,
-             timeout_milliseconds / 1000);
+        dmsg(
+            D_SEMAPHORE_LOW,
+            "Attempting to lock Win32 semaphore '%s' prior to net shell command (timeout = %d sec)",
+            s->name, timeout_milliseconds / 1000);
         status = WaitForSingleObject(s->hand, timeout_milliseconds);
         if (status == WAIT_FAILED)
         {
@@ -796,8 +794,7 @@ semaphore_lock(struct semaphore *s, int timeout_milliseconds)
         else
         {
             dmsg(D_SEMAPHORE, "Wait on Win32 semaphore '%s' timed out after %d milliseconds",
-                 s->name,
-                 timeout_milliseconds);
+                 s->name, timeout_milliseconds);
         }
     }
     return ret;
@@ -812,8 +809,7 @@ semaphore_release(struct semaphore *s)
         dmsg(D_SEMAPHORE, "Releasing Win32 semaphore '%s'", s->name);
         if (!ReleaseSemaphore(s->hand, 1, NULL))
         {
-            msg(M_WARN | M_ERRNO, "ReleaseSemaphore failed on Win32 semaphore '%s'",
-                s->name);
+            msg(M_WARN | M_ERRNO, "ReleaseSemaphore failed on Win32 semaphore '%s'", s->name);
         }
         s->locked = false;
     }
@@ -885,8 +881,9 @@ env_block(const struct env_set *es)
     char force_path[256];
     char *sysroot = get_win_sys_path();
 
-    if (!openvpn_snprintf(force_path, sizeof(force_path), "PATH=%s\\System32;%s;%s\\System32\\Wbem",
-                          sysroot, sysroot, sysroot))
+    if (snprintf(force_path, sizeof(force_path), "PATH=%s\\System32;%s;%s\\System32\\Wbem",
+                 sysroot, sysroot, sysroot)
+        >= sizeof(force_path))
     {
         msg(M_WARN, "env_block: default path truncated to %s", force_path);
     }
@@ -904,9 +901,9 @@ env_block(const struct env_set *es)
             nchars += strlen(e->string) + 1;
         }
 
-        nchars += strlen(force_path)+1;
+        nchars += strlen(force_path) + 1;
 
-        ret = (char *) malloc(nchars);
+        ret = (char *)malloc(nchars);
         check_malloc_return(ret);
 
         p = ret;
@@ -917,7 +914,7 @@ env_block(const struct env_set *es)
                 strcpy(p, e->string);
                 p += strlen(e->string) + 1;
             }
-            if (strncmp(e->string, "PATH=", 5 ) == 0)
+            if (strncmp(e->string, "PATH=", 5) == 0)
             {
                 path_seen = true;
             }
@@ -926,8 +923,8 @@ env_block(const struct env_set *es)
         /* make sure PATH is set */
         if (!path_seen)
         {
-            msg( M_INFO, "env_block: add %s", force_path );
-            strcpy( p, force_path );
+            msg(M_INFO, "env_block: add %s", force_path);
+            strcpy(p, force_path);
             p += strlen(force_path) + 1;
         }
 
@@ -973,7 +970,7 @@ wide_cmd_line(const struct argv *a, struct gc_arena *gc)
     {
         const char *arg = a->argv[i];
         strcpy(work, arg);
-        string_mod(work, CC_PRINT, CC_DOUBLE_QUOTE|CC_CRLF, '_');
+        string_mod(work, CC_PRINT, CC_DOUBLE_QUOTE | CC_CRLF, '_');
         if (i)
         {
             buf_printf(&buf, " ");
@@ -1024,7 +1021,8 @@ openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned in
             start_info.dwFlags = STARTF_USESHOWWINDOW;
             start_info.wShowWindow = SW_HIDE;
 
-            if (CreateProcessW(cmd, cl, NULL, NULL, FALSE, proc_flags, env, NULL, &start_info, &proc_info))
+            if (CreateProcessW(cmd, cl, NULL, NULL, FALSE, proc_flags, env, NULL, &start_info,
+                               &proc_info))
             {
                 DWORD exit_status = 0;
                 CloseHandle(proc_info.hThread);
@@ -1035,13 +1033,13 @@ openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned in
                 }
                 else
                 {
-                    msg(M_WARN|M_ERRNO, "openvpn_execve: GetExitCodeProcess %ls failed", cmd);
+                    msg(M_WARN | M_ERRNO, "openvpn_execve: GetExitCodeProcess %ls failed", cmd);
                 }
                 CloseHandle(proc_info.hProcess);
             }
             else
             {
-                msg(M_WARN|M_ERRNO, "openvpn_execve: CreateProcess %ls failed", cmd);
+                msg(M_WARN | M_ERRNO, "openvpn_execve: CreateProcess %ls failed", cmd);
             }
             free(env);
             gc_free(&gc);
@@ -1082,7 +1080,8 @@ fork_to_self(const char *cmdline)
     status = GetModuleFileName(NULL, self_exe, sizeof(self_exe));
     if (status == 0 || status == sizeof(self_exe))
     {
-        msg(M_WARN|M_ERRNO, "fork_to_self: CreateProcess failed: cannot get module name via GetModuleFileName");
+        msg(M_WARN | M_ERRNO,
+            "fork_to_self: CreateProcess failed: cannot get module name via GetModuleFileName");
         goto done;
     }
 
@@ -1099,7 +1098,7 @@ fork_to_self(const char *cmdline)
     }
     else
     {
-        msg(M_WARN|M_ERRNO, "fork_to_self: CreateProcess failed: %s", cmdline);
+        msg(M_WARN | M_ERRNO, "fork_to_self: CreateProcess failed: %s", cmdline);
     }
 
 done:
@@ -1132,77 +1131,26 @@ set_win_sys_path_via_env(struct env_set *es)
     }
     if (status > sizeof(buf) - 1)
     {
-        msg(M_FATAL, "String overflow attempting to read environmental variable %s", SYS_PATH_ENV_VAR_NAME);
+        msg(M_FATAL, "String overflow attempting to read environmental variable %s",
+            SYS_PATH_ENV_VAR_NAME);
     }
     set_win_sys_path(buf, es);
 }
 
-
-const char *
-win_get_tempdir(void)
-{
-    static char tmpdir[MAX_PATH];
-    WCHAR wtmpdir[MAX_PATH];
-
-    if (!GetTempPathW(_countof(wtmpdir), wtmpdir))
-    {
-        /* Warn if we can't find a valid temporary directory, which should
-         * be unlikely.
-         */
-        msg(M_WARN, "Could not find a suitable temporary directory."
-            " (GetTempPath() failed).  Consider using --tmp-dir");
-        return NULL;
-    }
-
-    if (WideCharToMultiByte(CP_UTF8, 0, wtmpdir, -1, NULL, 0, NULL, NULL) > sizeof(tmpdir))
-    {
-        msg(M_WARN, "Could not get temporary directory. Path is too long."
-            "  Consider using --tmp-dir");
-        return NULL;
-    }
-
-    WideCharToMultiByte(CP_UTF8, 0, wtmpdir, -1, tmpdir, sizeof(tmpdir), NULL, NULL);
-    return tmpdir;
-}
-
 static bool
-win_block_dns_service(bool add, int index, const HANDLE pipe)
+win_get_exe_path(PWCHAR path, DWORD size)
 {
-    bool ret = false;
-    ack_message_t ack;
-    struct gc_arena gc = gc_new();
-
-    block_dns_message_t data = {
-        .header = {
-            (add ? msg_add_block_dns : msg_del_block_dns),
-            sizeof(block_dns_message_t),
-            0
-        },
-        .iface = { .index = index, .name = "" }
-    };
-
-    if (!send_msg_iservice(pipe, &data, sizeof(data), &ack, "Block_DNS"))
+    DWORD status = GetModuleFileNameW(NULL, path, size);
+    if (status == 0 || status == size)
     {
-        goto out;
+        msg(M_WARN | M_ERRNO, "cannot get executable path");
+        return false;
     }
-
-    if (ack.error_number != NO_ERROR)
-    {
-        msg(M_WARN, "Block_DNS: %s block dns filters using service failed: %s [status=0x%x if_index=%d]",
-            (add ? "adding" : "deleting"), strerror_win32(ack.error_number, &gc),
-            ack.error_number, data.iface.index);
-        goto out;
-    }
-
-    ret = true;
-    msg(M_INFO, "%s outside dns using service succeeded.", (add ? "Blocking" : "Unblocking"));
-out:
-    gc_free(&gc);
-    return ret;
+    return true;
 }
 
 static void
-block_dns_msg_handler(DWORD err, const char *msg)
+win_wfp_msg_handler(DWORD err, const char *msg)
 {
     struct gc_arena gc = gc_new();
 
@@ -1212,15 +1160,47 @@ block_dns_msg_handler(DWORD err, const char *msg)
     }
     else
     {
-        msg(M_WARN, "Error in add_block_dns_filters(): %s : %s [status=0x%lx]",
-            msg, strerror_win32(err, &gc), err);
+        msg(M_WARN, "Error in WFP: %s : %s [status=0x%lx]", msg, strerror_win32(err, &gc), err);
     }
 
     gc_free(&gc);
 }
 
+static bool
+win_wfp_block_service(bool add, bool dns_only, int index, const HANDLE pipe)
+{
+    bool ret = false;
+    ack_message_t ack;
+    struct gc_arena gc = gc_new();
+
+    wfp_block_message_t data = { .header = { (add ? msg_add_wfp_block : msg_del_wfp_block),
+                                             sizeof(wfp_block_message_t), 0 },
+                                 .flags = dns_only ? wfp_block_dns : wfp_block_local,
+                                 .iface = { .index = index, .name = "" } };
+
+    if (!send_msg_iservice(pipe, &data, sizeof(data), &ack, "WFP block"))
+    {
+        goto out;
+    }
+
+    if (ack.error_number != NO_ERROR)
+    {
+        msg(M_WARN,
+            "WFP block: %s block filters using service failed: %s [status=0x%x if_index=%d]",
+            (add ? "adding" : "deleting"), strerror_win32(ack.error_number, &gc), ack.error_number,
+            data.iface.index);
+        goto out;
+    }
+
+    ret = true;
+    msg(M_INFO, "%s WFP block filters using service succeeded.", (add ? "Adding" : "Deleting"));
+out:
+    gc_free(&gc);
+    return ret;
+}
+
 bool
-win_wfp_block_dns(const NET_IFINDEX index, const HANDLE msg_channel)
+win_wfp_block(const NET_IFINDEX index, const HANDLE msg_channel, BOOL dns_only)
 {
     WCHAR openvpnpath[MAX_PATH];
     bool ret = false;
@@ -1228,20 +1208,19 @@ win_wfp_block_dns(const NET_IFINDEX index, const HANDLE msg_channel)
 
     if (msg_channel)
     {
-        dmsg(D_LOW, "Using service to add block dns filters");
-        ret = win_block_dns_service(true, index, msg_channel);
+        dmsg(D_LOW, "Using service to add WFP block filters");
+        ret = win_wfp_block_service(true, dns_only, index, msg_channel);
         goto out;
     }
 
-    status = GetModuleFileNameW(NULL, openvpnpath, _countof(openvpnpath));
-    if (status == 0 || status == _countof(openvpnpath))
+    ret = win_get_exe_path(openvpnpath, _countof(openvpnpath));
+    if (ret == false)
     {
-        msg(M_WARN|M_ERRNO, "block_dns: cannot get executable path");
         goto out;
     }
 
-    status = add_block_dns_filters(&m_hEngineHandle, index, openvpnpath,
-                                   block_dns_msg_handler);
+    status =
+        add_wfp_block_filters(&m_hEngineHandle, index, openvpnpath, win_wfp_msg_handler, dns_only);
     if (status == 0)
     {
         int is_auto = 0;
@@ -1255,10 +1234,10 @@ win_wfp_block_dns(const NET_IFINDEX index, const HANDLE msg_channel)
         {
             tap_metric_v6 = 0;
         }
-        status = set_interface_metric(index, AF_INET, BLOCK_DNS_IFACE_METRIC);
+        status = set_interface_metric(index, AF_INET, WFP_BLOCK_IFACE_METRIC);
         if (!status)
         {
-            set_interface_metric(index, AF_INET6, BLOCK_DNS_IFACE_METRIC);
+            set_interface_metric(index, AF_INET6, WFP_BLOCK_IFACE_METRIC);
         }
     }
 
@@ -1276,12 +1255,12 @@ win_wfp_uninit(const NET_IFINDEX index, const HANDLE msg_channel)
 
     if (msg_channel)
     {
-        msg(D_LOW, "Using service to delete block dns filters");
-        win_block_dns_service(false, index, msg_channel);
+        msg(D_LOW, "Using service to delete WFP block filters");
+        win_wfp_block_service(false, false, index, msg_channel);
     }
     else
     {
-        delete_block_dns_filters(m_hEngineHandle);
+        delete_wfp_block_filters(m_hEngineHandle);
         m_hEngineHandle = NULL;
         if (tap_metric_v4 >= 0)
         {
@@ -1296,43 +1275,8 @@ win_wfp_uninit(const NET_IFINDEX index, const HANDLE msg_channel)
     return true;
 }
 
-int
-win32_version_info(void)
+typedef enum
 {
-    if (!IsWindowsXPOrGreater())
-    {
-        msg(M_FATAL, "Error: Windows version must be XP or greater.");
-    }
-
-    if (!IsWindowsVistaOrGreater())
-    {
-        return WIN_XP;
-    }
-
-    if (!IsWindows7OrGreater())
-    {
-        return WIN_VISTA;
-    }
-
-    if (!IsWindows8OrGreater())
-    {
-        return WIN_7;
-    }
-
-    if (!IsWindows8Point1OrGreater())
-    {
-        return WIN_8;
-    }
-
-    if (!IsWindows10OrGreater())
-    {
-        return WIN_8_1;
-    }
-
-    return WIN_10;
-}
-
-typedef enum {
     ARCH_X86,
     ARCH_AMD64,
     ARCH_ARM64,
@@ -1346,9 +1290,9 @@ win32_get_arch(arch_t *process_arch, arch_t *host_arch)
     *process_arch = ARCH_UNKNOWN;
     *host_arch = ARCH_NATIVE;
 
-    typedef BOOL (WINAPI *is_wow64_process2_t)(HANDLE, USHORT *, USHORT *);
-    is_wow64_process2_t is_wow64_process2 = (is_wow64_process2_t)
-                                            GetProcAddress(GetModuleHandle("Kernel32.dll"), "IsWow64Process2");
+    typedef BOOL(WINAPI * is_wow64_process2_t)(HANDLE, USHORT *, USHORT *);
+    is_wow64_process2_t is_wow64_process2 =
+        (is_wow64_process2_t)GetProcAddress(GetModuleHandle("Kernel32.dll"), "IsWow64Process2");
 
     USHORT process_machine = 0;
     USHORT native_machine = 0;
@@ -1360,8 +1304,7 @@ win32_get_arch(arch_t *process_arch, arch_t *host_arch)
     if (is_wow64_process2)
     {
         /* this could be amd64 on arm64 */
-        BOOL is_wow64 = is_wow64_process2(GetCurrentProcess(),
-                                          &process_machine, &native_machine);
+        BOOL is_wow64 = is_wow64_process2(GetCurrentProcess(), &process_machine, &native_machine);
         if (is_wow64 && native_machine == IMAGE_FILE_MACHINE_ARM64)
         {
             *host_arch = ARCH_ARM64;
@@ -1373,8 +1316,7 @@ win32_get_arch(arch_t *process_arch, arch_t *host_arch)
     if (is_wow64_process2)
     {
         /* check if we're running on arm64 or amd64 machine */
-        BOOL is_wow64 = is_wow64_process2(GetCurrentProcess(),
-                                          &process_machine, &native_machine);
+        BOOL is_wow64 = is_wow64_process2(GetCurrentProcess(), &process_machine, &native_machine);
         if (is_wow64)
         {
             switch (native_machine)
@@ -1433,52 +1375,40 @@ win32_print_arch(arch_t arch, struct buffer *out)
     }
 }
 
+typedef LONG(WINAPI *RtlGetVersionPtr)(PRTL_OSVERSIONINFOW);
+
 const char *
-win32_version_string(struct gc_arena *gc, bool add_name)
+win32_version_string(struct gc_arena *gc)
 {
-    int version = win32_version_info();
-    struct buffer out = alloc_buf_gc(256, gc);
-
-    switch (version)
+    HMODULE hMod = GetModuleHandleW(L"ntdll.dll");
+    if (!hMod)
     {
-        case WIN_XP:
-            buf_printf(&out, "5.1%s", add_name ? " (Windows XP)" : "");
-            break;
-
-        case WIN_VISTA:
-            buf_printf(&out, "6.0%s", add_name ? " (Windows Vista)" : "");
-            break;
-
-        case WIN_7:
-            buf_printf(&out, "6.1%s", add_name ? " (Windows 7)" : "");
-            break;
-
-        case WIN_8:
-            buf_printf(&out, "6.2%s", add_name ? " (Windows 8)" : "");
-            break;
-
-        case WIN_8_1:
-            buf_printf(&out, "6.3%s", add_name ? " (Windows 8.1)" : "");
-            break;
-
-        case WIN_10:
-            buf_printf(&out, "10.0%s", add_name ? " (Windows 10 or greater)" : "");
-            break;
-
-        default:
-            msg(M_NONFATAL, "Unknown Windows version: %d", version);
-            buf_printf(&out, "0.0%s", add_name ? " (unknown)" : "");
-            break;
+        return "N/A";
     }
 
-    buf_printf(&out, ", ");
+    RtlGetVersionPtr fn = (RtlGetVersionPtr)GetProcAddress(hMod, "RtlGetVersion");
+    if (!fn)
+    {
+        return "N/A";
+    }
+
+    RTL_OSVERSIONINFOW rovi = { 0 };
+    rovi.dwOSVersionInfoSize = sizeof(rovi);
+    if (fn(&rovi) != 0)
+    {
+        return "N/A";
+    }
+
+    struct buffer out = alloc_buf_gc(256, gc);
+
+    buf_printf(&out, "%lu.%lu.%lu", rovi.dwMajorVersion, rovi.dwMinorVersion, rovi.dwBuildNumber);
+
+    buf_printf(&out, ",");
 
     arch_t process_arch, host_arch;
     win32_get_arch(&process_arch, &host_arch);
     win32_print_arch(process_arch, &out);
 
-    buf_printf(&out, " executable");
-
     if (host_arch != ARCH_NATIVE)
     {
         buf_printf(&out, " running on ");
@@ -1490,18 +1420,16 @@ win32_version_string(struct gc_arena *gc, bool add_name)
 }
 
 bool
-send_msg_iservice(HANDLE pipe, const void *data, size_t size,
-                  ack_message_t *ack, const char *context)
+send_msg_iservice(HANDLE pipe, const void *data, DWORD size, ack_message_t *ack,
+                  const char *context)
 {
     struct gc_arena gc = gc_new();
     DWORD len;
     bool ret = true;
 
-    if (!WriteFile(pipe, data, size, &len, NULL)
-        || !ReadFile(pipe, ack, sizeof(*ack), &len, NULL))
+    if (!WriteFile(pipe, data, size, &len, NULL) || !ReadFile(pipe, ack, sizeof(*ack), &len, NULL))
     {
-        msg(M_WARN, "%s: could not talk to service: %s [%lu]",
-            context ? context : "Unknown",
+        msg(M_WARN, "%s: could not talk to service: %s [%lu]", context ? context : "Unknown",
             strerror_win32(GetLastError(), &gc), GetLastError());
         ret = false;
     }
@@ -1510,27 +1438,12 @@ send_msg_iservice(HANDLE pipe, const void *data, size_t size,
     return ret;
 }
 
-bool
-openvpn_swprintf(wchar_t *const str, const size_t size, const wchar_t *const format, ...)
-{
-    va_list arglist;
-    int len = -1;
-    if (size > 0)
-    {
-        va_start(arglist, format);
-        len = vswprintf(str, size, format, arglist);
-        va_end(arglist);
-        str[size - 1] = L'\0';
-    }
-    return (len >= 0 && len < size);
-}
-
 bool
 get_openvpn_reg_value(const WCHAR *key, WCHAR *value, DWORD size)
 {
     WCHAR reg_path[256];
     HKEY hkey;
-    openvpn_swprintf(reg_path, _countof(reg_path), L"SOFTWARE\\" PACKAGE_NAME);
+    swprintf(reg_path, _countof(reg_path), L"SOFTWARE\\" PACKAGE_NAME);
 
     LONG status = RegOpenKeyExW(HKEY_LOCAL_MACHINE, reg_path, 0, KEY_READ, &hkey);
     if (status != ERROR_SUCCESS)
@@ -1546,7 +1459,7 @@ get_openvpn_reg_value(const WCHAR *key, WCHAR *value, DWORD size)
 }
 
 static void
-set_openssl_env_vars()
+set_openssl_env_vars(void)
 {
     const WCHAR *ssl_fallback_dir = L"C:\\Windows\\System32";
 
@@ -1556,7 +1469,7 @@ set_openssl_env_vars()
         /* if we cannot find installation path from the registry,
          * use Windows directory as a fallback
          */
-        openvpn_swprintf(install_path, _countof(install_path), L"%ls", ssl_fallback_dir);
+        swprintf(install_path, _countof(install_path), L"%ls", ssl_fallback_dir);
     }
 
     if ((install_path[wcslen(install_path) - 1]) == L'\\')
@@ -1564,14 +1477,13 @@ set_openssl_env_vars()
         install_path[wcslen(install_path) - 1] = L'\0';
     }
 
-    static struct {
+    static struct
+    {
         WCHAR *name;
         WCHAR *value;
-    } ossl_env[] = {
-        {L"OPENSSL_CONF", L"openssl.cnf"},
-        {L"OPENSSL_ENGINES", L"engines"},
-        {L"OPENSSL_MODULES", L"modules"}
-    };
+    } ossl_env[] = { { L"OPENSSL_CONF", L"openssl.cnf" },
+                     { L"OPENSSL_ENGINES", L"engines" },
+                     { L"OPENSSL_MODULES", L"modules" } };
 
     for (size_t i = 0; i < SIZE(ossl_env); ++i)
     {
@@ -1580,8 +1492,8 @@ set_openssl_env_vars()
         _wgetenv_s(&size, NULL, 0, ossl_env[i].name);
         if (size == 0)
         {
-            WCHAR val[MAX_PATH] = {0};
-            openvpn_swprintf(val, _countof(val), L"%ls\\ssl\\%ls", install_path, ossl_env[i].value);
+            WCHAR val[MAX_PATH] = { 0 };
+            swprintf(val, _countof(val), L"%ls\\ssl\\%ls", install_path, ossl_env[i].value);
             _wputenv_s(ossl_env[i].name, val);
         }
     }
@@ -1601,7 +1513,7 @@ win32_sleep(const int n)
     {
         if (n > 0)
         {
-            Sleep(n*1000);
+            Sleep(n * 1000);
         }
         return;
     }
@@ -1611,9 +1523,9 @@ win32_sleep(const int n)
 
     while (expire >= now)
     {
-        DWORD status = WaitForSingleObject(win32_signal.in.read, (expire-now)*1000);
-        if ((status == WAIT_OBJECT_0 && win32_signal_get(&win32_signal))
-            || status == WAIT_TIMEOUT)
+        DWORD wait_ms = (DWORD)((expire - now) * 1000);
+        DWORD status = WaitForSingleObject(win32_signal.in.read, wait_ms);
+        if ((status == WAIT_OBJECT_0 && win32_signal_get(&win32_signal)) || status == WAIT_TIMEOUT)
         {
             return;
         }
@@ -1624,7 +1536,7 @@ win32_sleep(const int n)
         {
             if (expire > now)
             {
-                Sleep((expire-now)*1000);
+                Sleep((DWORD)((expire - now) * 1000));
             }
             return;
         }
@@ -1676,8 +1588,8 @@ plugin_in_trusted_dir(const WCHAR *plugin_path)
     }
 
     /* Check if the plugin path resides within the plugin/install directory */
-    if ((wcslen(normalized_plugin_dir) > 0) && (wcsnicmp(normalized_plugin_dir,
-                                                         plugin_path, wcslen(normalized_plugin_dir)) == 0))
+    if ((wcslen(normalized_plugin_dir) > 0)
+        && (wcsnicmp(normalized_plugin_dir, plugin_path, wcslen(normalized_plugin_dir)) == 0))
     {
         return true;
     }
@@ -1687,7 +1599,7 @@ plugin_in_trusted_dir(const WCHAR *plugin_path)
 }
 
 bool
-protect_buffer_win32(char *buf, size_t len)
+protect_buffer_win32(char *buf, DWORD len)
 {
     bool ret;
     if (len % CRYPTPROTECTMEMORY_BLOCK_SIZE)
@@ -1705,7 +1617,7 @@ protect_buffer_win32(char *buf, size_t len)
 }
 
 bool
-unprotect_buffer_win32(char *buf, size_t len)
+unprotect_buffer_win32(char *buf, DWORD len)
 {
     bool ret;
     if (len % CRYPTPROTECTMEMORY_BLOCK_SIZE)
