@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2026 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,28 +17,30 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef _WIN32
 #ifndef OPENVPN_WIN32_H
 #define OPENVPN_WIN32_H
 
-#include <winioctl.h>
+#include <iphlpapi.h>
+#include <ws2tcpip.h>
 
+#include "syshead.h"
 #include "mtu.h"
 #include "openvpn-msg.h"
 #include "argv.h"
 #include "win32-util.h"
 
 /* location of executables */
-#define SYS_PATH_ENV_VAR_NAME "SystemRoot"  /* environmental variable name that normally contains the system path */
-#define NETSH_PATH_SUFFIX     "\\system32\\netsh.exe"
-#define WIN_ROUTE_PATH_SUFFIX "\\system32\\route.exe"
+#define SYS_PATH_ENV_VAR_NAME \
+    "SystemRoot" /* environmental variable name that normally contains the system path */
+#define NETSH_PATH_SUFFIX        "\\system32\\netsh.exe"
+#define WIN_ROUTE_PATH_SUFFIX    "\\system32\\route.exe"
 #define WIN_IPCONFIG_PATH_SUFFIX "\\system32\\ipconfig.exe"
-#define WIN_NET_PATH_SUFFIX "\\system32\\net.exe"
-#define WMIC_PATH_SUFFIX "\\system32\\wbem\\wmic.exe"
+#define WIN_NET_PATH_SUFFIX      "\\system32\\net.exe"
+#define POWERSHELL_PATH_SUFFIX   "\\system32\\WindowsPowerShell\\v1.0\\powershell.exe"
 
 /*
  * Win32-specific OpenVPN code, targeted at the mingw
@@ -74,7 +76,8 @@ struct window_title
     char old_window_title[256];
 };
 
-struct rw_handle {
+struct rw_handle
+{
     HANDLE read;
     HANDLE write;
 };
@@ -83,8 +86,8 @@ struct rw_handle {
  * Event-based notification of incoming TCP connections
  */
 
-#define NE32_PERSIST_EVENT (1<<0)
-#define NE32_WRITE_EVENT   (1<<1)
+#define NE32_PERSIST_EVENT (1 << 0)
+#define NE32_WRITE_EVENT   (1 << 1)
 
 static inline bool
 defined_net_event_win32(const struct rw_handle *event)
@@ -92,7 +95,8 @@ defined_net_event_win32(const struct rw_handle *event)
     return event->read != NULL;
 }
 
-void init_net_event_win32(struct rw_handle *event, long network_events, socket_descriptor_t sd, unsigned int flags);
+void init_net_event_win32(struct rw_handle *event, long network_events, socket_descriptor_t sd,
+                          unsigned int flags);
 
 long reset_net_event_win32(struct rw_handle *event, socket_descriptor_t sd);
 
@@ -148,7 +152,8 @@ net_event_win32_clear_selected_events(struct net_event_win32 *ne, long selected_
 /*
  * Signal handling
  */
-struct win32_signal {
+struct win32_signal
+{
 #define WSO_MODE_UNDEF   0
 #define WSO_MODE_SERVICE 1
 #define WSO_MODE_CONSOLE 2
@@ -168,10 +173,8 @@ void win32_signal_clear(struct win32_signal *ws);
 #define WSO_FORCE_SERVICE 1
 #define WSO_FORCE_CONSOLE 2
 
-void win32_signal_open(struct win32_signal *ws,
-                       int force,  /* set to WSO force parm */
-                       const char *exit_event_name,
-                       bool exit_event_initial_state);
+void win32_signal_open(struct win32_signal *ws, int force, /* set to WSO force parm */
+                       const char *exit_event_name, bool exit_event_initial_state);
 
 void win32_signal_close(struct win32_signal *ws);
 
@@ -197,17 +200,19 @@ void window_title_generate(const char *title);
  * We try to do all Win32 I/O using overlapped
  * (i.e. asynchronous) I/O for a performance win.
  */
-struct overlapped_io {
+struct overlapped_io
+{
 #define IOSTATE_INITIAL          0
-#define IOSTATE_QUEUED           1  /* overlapped I/O has been queued */
-#define IOSTATE_IMMEDIATE_RETURN 2  /* I/O function returned immediately without queueing */
+#define IOSTATE_QUEUED           1 /* overlapped I/O has been queued */
+#define IOSTATE_IMMEDIATE_RETURN 2 /* I/O function returned immediately without queueing */
     int iostate;
     OVERLAPPED overlapped;
     DWORD size;
     DWORD flags;
     int status;
     bool addr_defined;
-    union {
+    union
+    {
         struct sockaddr_in addr;
         struct sockaddr_in6 addr6;
     };
@@ -216,9 +221,7 @@ struct overlapped_io {
     struct buffer buf;
 };
 
-void overlapped_io_init(struct overlapped_io *o,
-                        const struct frame *frame,
-                        BOOL event_state);
+void overlapped_io_init(struct overlapped_io *o, const struct frame *frame, BOOL event_state);
 
 void overlapped_io_close(struct overlapped_io *o);
 
@@ -286,49 +289,32 @@ char *get_win_sys_path(void);
 /* call self in a subprocess */
 void fork_to_self(const char *cmdline);
 
-/* Find temporary directory */
-const char *win_get_tempdir(void);
-
-bool win_wfp_block_dns(const NET_IFINDEX index, const HANDLE msg_channel);
+bool win_wfp_block(const NET_IFINDEX index, const HANDLE msg_channel, BOOL dns_only);
 
 bool win_wfp_uninit(const NET_IFINDEX index, const HANDLE msg_channel);
 
-#define WIN_XP    0
-#define WIN_VISTA 1
-#define WIN_7     2
-#define WIN_8     3
-#define WIN_8_1   4
-#define WIN_10    5
-
-int win32_version_info(void);
-
-/*
- * String representation of Windows version number and name, see
- * https://msdn.microsoft.com/en-us/library/windows/desktop/ms724832(v=vs.85).aspx
+/**
+ * @brief Get Windows version string with architecture info.
+ *
+ * @param gc gc arena to allocate string.
+ * @return Version string, or "N/A" on failure.
  */
-const char *win32_version_string(struct gc_arena *gc, bool add_name);
+const char *win32_version_string(struct gc_arena *gc);
 
-/*
- * Send the |size| bytes in buffer |data| to the interactive service |pipe|
- * and read the result in |ack|. Returns false on communication error.
- * The string in |context| is used to prefix error messages.
+/**
+ * Send the \p size bytes in buffer \p data to the interactive service \p pipe
+ * and read the result in \p ack.
+ * The string in \p context is used to prefix error messages.
+ *
+ * @return true on success, false on communication error
  */
-bool send_msg_iservice(HANDLE pipe, const void *data, size_t size,
-                       ack_message_t *ack, const char *context);
+bool send_msg_iservice(HANDLE pipe, const void *data, DWORD size, ack_message_t *ack,
+                       const char *context);
 
 /*
  * Attempt to simulate fork/execve on Windows
  */
-int
-openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned int flags);
-
-/*
- * openvpn_swprintf() is currently only used by Windows code paths
- * and when enabled for all platforms it will currently break older
- * OpenBSD versions lacking vswprintf(3) support in their libc.
- */
-bool
-openvpn_swprintf(wchar_t *const str, const size_t size, const wchar_t *const format, ...);
+int openvpn_execve(const struct argv *a, const struct env_set *es, const unsigned int flags);
 
 /* Sleep that can be interrupted by signals and exit event */
 void win32_sleep(const int n);
@@ -341,8 +327,7 @@ void win32_sleep(const int n);
  * @param size Size of `value` buffer in bytes.
  * @return `true` if successful, `false` otherwise.
  */
-bool
-get_openvpn_reg_value(const WCHAR *key, WCHAR *value, DWORD size);
+bool get_openvpn_reg_value(const WCHAR *key, WCHAR *value, DWORD size);
 
 /**
  * @brief Checks if a plugin is located in a trusted directory.
@@ -357,8 +342,7 @@ get_openvpn_reg_value(const WCHAR *key, WCHAR *value, DWORD size);
  * @param plugin_path Normalized path to the plugin.
  * @return \c true if the plugin is in a trusted directory and not a UNC path; \c false otherwise.
  */
-bool
-plugin_in_trusted_dir(const WCHAR *plugin_path);
+bool plugin_in_trusted_dir(const WCHAR *plugin_path);
 
 /**
  * Encrypt a region of memory using CryptProtectMemory()
@@ -368,8 +352,7 @@ plugin_in_trusted_dir(const WCHAR *plugin_path);
  * - len   number of bytes to encrypt -- must be a multiple of
  *         CRYPTPROTECTMEMORY_BLOCK_SIZE = 16
  */
-bool
-protect_buffer_win32(char *buf, size_t len);
+bool protect_buffer_win32(char *buf, DWORD len);
 
 /**
  * Decrypt a previously encrypted region of memory using CryptUnProtectMemory()
@@ -379,8 +362,7 @@ protect_buffer_win32(char *buf, size_t len);
  * - len   number of bytes to encrypt -- must be a multiple of
  *         CRYPTPROTECTMEMORY_BLOCK_SIZE = 16
  */
-bool
-unprotect_buffer_win32(char *buf, size_t len);
+bool unprotect_buffer_win32(char *buf, DWORD len);
 
 #endif /* ifndef OPENVPN_WIN32_H */
 #endif /* ifdef _WIN32 */
