@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2016-2024 Selva Nair <selva.nair@gmail.com>
+ *  Copyright (C) 2016-2026 Selva Nair <selva.nair@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -17,18 +17,17 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "validate.h"
 
 #include <lmaccess.h>
 #include <shlwapi.h>
+#include <pathcch.h>
 #include <lm.h>
 
-static const WCHAR *white_list[] =
-{
+static const WCHAR *white_list[] = {
     L"auth-retry",
     L"config",
     L"log",
@@ -47,7 +46,7 @@ static const WCHAR *white_list[] =
     L"pull-filter",
     L"script-security",
 
-    NULL                                /* last value */
+    NULL /* last value */
 };
 
 static BOOL IsUserInGroup(PSID sid, const PTOKEN_GROUPS groups, const WCHAR *group_name);
@@ -55,36 +54,31 @@ static BOOL IsUserInGroup(PSID sid, const PTOKEN_GROUPS groups, const WCHAR *gro
 static PTOKEN_GROUPS GetTokenGroups(const HANDLE token);
 
 /*
- * Check workdir\fname is inside config_dir
- * The logic here is simple: we may reject some valid paths if ..\ is in any of the strings
+ * Check that config path is inside config_dir
+ * The logic here is simple: if the path isn't prefixed with config_dir it's rejected
  */
 static BOOL
 CheckConfigPath(const WCHAR *workdir, const WCHAR *fname, const settings_t *s)
 {
-    WCHAR tmp[MAX_PATH];
-    const WCHAR *config_file = NULL;
-    const WCHAR *config_dir = NULL;
+    HRESULT res;
+    WCHAR config_path[MAX_PATH];
 
-    /* convert fname to full path */
-    if (PathIsRelativeW(fname) )
+    /* fname = stdin is special: do not treat it as a relative path */
+    if (wcscmp(fname, L"stdin") == 0)
     {
-        openvpn_swprintf(tmp, _countof(tmp), L"%ls\\%ls", workdir, fname);
-        config_file = tmp;
+        return FALSE;
+    }
+    /* convert fname to full canonical path */
+    if (PathIsRelativeW(fname))
+    {
+        res = PathCchCombine(config_path, _countof(config_path), workdir, fname);
     }
     else
     {
-        config_file = fname;
+        res = PathCchCanonicalize(config_path, _countof(config_path), fname);
     }
 
-    config_dir = s->config_dir;
-
-    if (wcsncmp(config_dir, config_file, wcslen(config_dir)) == 0
-        && wcsstr(config_file + wcslen(config_dir), L"..") == NULL)
-    {
-        return TRUE;
-    }
-
-    return FALSE;
+    return res == S_OK && wcsnicmp(config_path, s->config_dir, wcslen(s->config_dir)) == 0;
 }
 
 
@@ -129,7 +123,7 @@ GetBuiltinAdminGroupName(WCHAR *name, DWORD nlen)
         return FALSE;
     }
 
-    b = CreateWellKnownSid(WinBuiltinAdministratorsSid, NULL, admin_sid,  &sid_size);
+    b = CreateWellKnownSid(WinBuiltinAdministratorsSid, NULL, admin_sid, &sid_size);
     if (b)
     {
         b = LookupAccountSidW(NULL, admin_sid, name, &nlen, domain, &dlen, &snu);
@@ -140,12 +134,9 @@ GetBuiltinAdminGroupName(WCHAR *name, DWORD nlen)
     return b;
 }
 
-/*
- * Check whether user is a member of Administrators group or
- * the group specified in ovpn_admin_group
- */
 BOOL
-IsAuthorizedUser(PSID sid, const HANDLE token, const WCHAR *ovpn_admin_group)
+IsAuthorizedUser(PSID sid, const HANDLE token, const WCHAR *ovpn_admin_group,
+                 const WCHAR *ovpn_service_user)
 {
     const WCHAR *admin_group[2];
     WCHAR username[MAX_NAME];
@@ -158,19 +149,26 @@ IsAuthorizedUser(PSID sid, const HANDLE token, const WCHAR *ovpn_admin_group)
     /* Get username */
     if (!LookupAccountSidW(NULL, sid, username, &len, domain, &len, &sid_type))
     {
-        MsgToEventLog(M_SYSERR, TEXT("LookupAccountSid"));
+        MsgToEventLog(M_SYSERR, L"LookupAccountSid");
         /* not fatal as this is now used only for logging */
         username[0] = '\0';
         domain[0] = '\0';
     }
 
+    /* is this service account? */
+    if ((wcscmp(username, ovpn_service_user) == 0) && (wcscmp(domain, L"NT SERVICE") == 0))
+    {
+        return TRUE;
+    }
+
     if (GetBuiltinAdminGroupName(sysadmin_group, _countof(sysadmin_group)))
     {
         admin_group[0] = sysadmin_group;
     }
     else
     {
-        MsgToEventLog(M_SYSERR, TEXT("Failed to get the name of Administrators group. Using the default."));
+        MsgToEventLog(M_SYSERR,
+                      L"Failed to get the name of Administrators group. Using the default.");
         /* use the default value */
         admin_group[0] = SYSTEM_ADMIN_GROUP;
     }
@@ -182,7 +180,8 @@ IsAuthorizedUser(PSID sid, const HANDLE token, const WCHAR *ovpn_admin_group)
         ret = IsUserInGroup(sid, token_groups, admin_group[i]);
         if (ret)
         {
-            MsgToEventLog(M_INFO, TEXT("Authorizing user '%ls@%ls' by virtue of membership in group '%ls'"),
+            MsgToEventLog(M_INFO,
+                          L"Authorizing user '%ls@%ls' by virtue of membership in group '%ls'",
                           username, domain, admin_group[i]);
             goto out;
         }
@@ -264,11 +263,11 @@ IsUserInGroup(PSID sid, const PTOKEN_GROUPS token_groups, const WCHAR *group_nam
     int nloop = 0; /* a counter used to not get stuck in the do .. while() */
 
     /* first check in the token groups */
-    if (token_groups && LookupSID(group_name, (PSID) grp_sid, _countof(grp_sid)))
+    if (token_groups && LookupSID(group_name, (PSID)grp_sid, _countof(grp_sid)))
     {
         for (DWORD i = 0; i < token_groups->GroupCount; ++i)
         {
-            if (EqualSid((PSID) grp_sid, token_groups->Groups[i].Sid))
+            if (EqualSid((PSID)grp_sid, token_groups->Groups[i].Sid))
             {
                 return TRUE;
             }
@@ -284,8 +283,8 @@ IsUserInGroup(PSID sid, const PTOKEN_GROUPS token_groups, const WCHAR *group_nam
     {
         DWORD nread, nmax;
         LOCALGROUP_MEMBERS_INFO_0 *members = NULL;
-        err = NetLocalGroupGetMembers(NULL, group_name, 0, (LPBYTE *) &members,
-                                      MAX_PREFERRED_LENGTH, &nread, &nmax, &resume);
+        err = NetLocalGroupGetMembers(NULL, group_name, 0, (LPBYTE *)&members, MAX_PREFERRED_LENGTH,
+                                      &nread, &nmax, &resume);
         if ((err != NERR_Success && err != ERROR_MORE_DATA))
         {
             break;
@@ -302,7 +301,7 @@ IsUserInGroup(PSID sid, const PTOKEN_GROUPS token_groups, const WCHAR *group_nam
     if (err != NERR_Success && err != NERR_GroupNotFound)
     {
         SetLastError(err);
-        MsgToEventLog(M_SYSERR, TEXT("In NetLocalGroupGetMembers for group '%ls'"), group_name);
+        MsgToEventLog(M_SYSERR, L"In NetLocalGroupGetMembers for group '%ls'", group_name);
     }
 
     return ret;
@@ -318,16 +317,13 @@ CheckOption(const WCHAR *workdir, int argc, WCHAR *argv[], const settings_t *s)
 {
     /* Do not modify argv or *argv -- ideally it should be const WCHAR *const *, but alas...*/
 
-    if (wcscmp(argv[0], L"--config") == 0
-        && argc > 1
-        && !CheckConfigPath(workdir, argv[1], s)
-        )
+    if (wcscmp(argv[0], L"--config") == 0 && argc > 1 && !CheckConfigPath(workdir, argv[1], s))
     {
         return FALSE;
     }
 
     /* option name starts at 2 characters from argv[i] */
-    if (OptionLookup(argv[0] + 2, white_list) == -1)   /* not found */
+    if (OptionLookup(argv[0] + 2, white_list) == -1) /* not found */
     {
         return FALSE;
     }
